# pylint: disable=no-self-use,invalid-name
import numpy as np
from numpy.testing import assert_almost_equal
import torch
from allennlp.common import Params
from allennlp.data import Vocabulary
from allennlp.modules.token_embedders import BagOfWordCountsTokenEmbedder
from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.checks import ConfigurationError

class TestBagOfWordCountsTokenEmbedder(AllenNlpTestCase):
    def setUp(self):
        super(TestBagOfWordCountsTokenEmbedder, self).setUp()
        self.vocab = Vocabulary()
        self.vocab.add_token_to_namespace("1")
        self.vocab.add_token_to_namespace("2")
        self.vocab.add_token_to_namespace("3")
        self.vocab.add_token_to_namespace("4")
        self.non_padded_vocab = Vocabulary(non_padded_namespaces=['tokens'])

    def test_forward_calculates_bow_properly(self):
        params = Params({})
        embedder = BagOfWordCountsTokenEmbedder.from_params(self.vocab, params=params)
        numpy_tensor = np.array([[2, 0], [3, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        numpy_tensor = np.array([[0, 0, 1, 0, 0, 0], [0, 0, 0, 1, 0, 0], [0, 0, 0, 0, 2, 0]])
        manual_output = torch.from_numpy(numpy_tensor).float()
        assert_almost_equal(embedder_output.data.numpy(), manual_output.data.numpy())

    def test_zeros_out_unknown_tokens(self):
        params = Params({"ignore_oov": True})
        embedder = BagOfWordCountsTokenEmbedder.from_params(self.vocab, params=params)
        numpy_tensor = np.array([[1, 5], [2, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        numpy_tensor = np.array([[0, 0, 0, 0, 0, 1], [0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 2, 0]])
        manual_output = torch.from_numpy(numpy_tensor).float()
        assert_almost_equal(embedder_output.data.numpy(), manual_output.data.numpy())

    def test_ignore_oov_should_fail_on_non_padded_vocab(self):
        params = Params({"ignore_oov": True})
        self.assertRaises(ConfigurationError,
                          BagOfWordCountsTokenEmbedder.from_params,
                          self.non_padded_vocab,
                          params)

    def test_projects_properly(self):
        params = Params({"projection_dim": 50})
        embedder = BagOfWordCountsTokenEmbedder.from_params(self.vocab, params=params)
        numpy_tensor = np.array([[1, 0], [1, 0], [4, 4]])
        inputs = torch.from_numpy(numpy_tensor)
        embedder_output = embedder(inputs)
        assert embedder_output.shape[1] == 50



# pylint: disable=invalid-name,no-self-use,protected-access
import argparse
import os
import json
import sys

from allennlp.commands import main, configure as configure_command
from allennlp.common.testing import AllenNlpTestCase
from allennlp.service.config_explorer import make_app


class TestConfigure(AllenNlpTestCase):
    def test_other_modules(self):
        # Create a new package in a temporary dir
        packagedir = self.TEST_DIR / 'configuretestpackage'
        packagedir.mkdir()  # pylint: disable=no-member
        (packagedir / '__init__.py').touch()  # pylint: disable=no-member

        # And add that directory to the path
        sys.path.insert(0, str(self.TEST_DIR))

        # Write out a duplicate predictor there, but registered under a different name.
        from allennlp.predictors import bidaf
        with open(bidaf.__file__) as f:
            code = f.read().replace("""@Predictor.register('machine-comprehension')""",
                                    """@Predictor.register('configure-test-predictor')""")

        with open(os.path.join(packagedir, 'predictor.py'), 'w') as f:
            f.write(code)

        app = None

        # Monkeypatch the run function
        def run_wizard(args: argparse.Namespace) -> None:
            nonlocal app

            app = make_app(args.include_package)
            app.testing = True

        configure_command._run_wizard = run_wizard

        sys.argv = ["run.py",      # executable
                    "configure",     # command
                    "--include-package", "configuretestpackage.predictor"]

        main()

        client = app.test_client()

        response = client.get('/api/config/?class=allennlp.predictors.predictor.Predictor&get_choices=true')
        data = json.loads(response.get_data())
        choices = data.get('choices', ())
        assert 'configuretestpackage.predictor.BidafPredictor' in choices



import json
from typing import Dict, Tuple, TYPE_CHECKING

import torch

from allennlp.common.checks import ConfigurationError
from allennlp.data import TokenIndexer, Token
from allennlp.modules.scalar_mix import ScalarMix
from allennlp.modules.token_embedders.token_embedder import TokenEmbedder
from allennlp.nn.util import remove_sentence_boundaries, get_text_field_mask, add_sentence_boundary_token_ids

# Importing at runtime results in a circular dependency.
if TYPE_CHECKING:
    from allennlp.models.language_model import LanguageModel


@TokenEmbedder.register('language_model_token_embedder')
class LanguageModelTokenEmbedder(TokenEmbedder):
    """
    Compute a single layer of representations from a (optionally bidirectional)
    language model. This is done by computing a learned scalar
    average of the layers from the LM. Typically the LM's weights
    will be fixed, but they can be fine tuned by setting ``requires_grad``.

    Parameters
    ----------
    archive_file : ``str``, required
        An archive file, typically model.tar.gz, from a LanguageModel.
        The contextualizer used by the LM must satisfy two requirements:

        1. It must have a num_layers field.
        2. It must take a boolean return_all_layers parameter in its constructor.

        See BidirectionalLanguageModelTransformer for their definitions.

    dropout : ``float``, optional.
        The dropout value to be applied to the representations.
    bos_eos_tokens : ``Tuple[str, str]``, optional (default=``("<S>", "</S>")``)
        These will be indexed and placed around the indexed tokens. Necessary if the language model
        was trained with them, but they were injected external to an indexer.
    remove_bos_eos: ``bool``, optional (default: True)
        Typically the provided token indexes will be augmented with begin-sentence and end-sentence
        tokens. (Alternatively, you can pass bos_eos_tokens.) If this flag is True the
        corresponding embeddings will be removed from the return values.

        Warning: This only removes a single start and single end token!
    requires_grad : ``bool``, optional (default: False)
        If True, compute gradient of bidirectional language model parameters for fine tuning.
    """
    def __init__(self,
                 archive_file: str,
                 dropout: float = None,
                 bos_eos_tokens: Tuple[str, str] = ("<S>", "</S>"),
                 remove_bos_eos: bool = True,
                 requires_grad: bool = False) -> None:
        super().__init__()

        overrides = {
                "model": {
                        "contextualizer": {
                                "return_all_layers": True
                        }
                }
        }

        # Import here to avoid circular dependency.
        from allennlp.models.archival import load_archive
        # Load LM and the associated config.
        archive = load_archive(archive_file, overrides=json.dumps(overrides))
        self._lm: LanguageModel = archive.model
        self._lm.delete_softmax()
        config = archive.config
        dict_config = config.as_dict(quiet=True)

        # Extract the name of the tokens that the LM was trained on.
        text_field_embedder = dict_config["model"]["text_field_embedder"]
        token_names = list(text_field_embedder["token_embedders"].keys())
        if len(token_names) != 1:
            # We don't currently support embedding with language models trained with multiple
            # embedded indices.
            #
            # Note: We only care about embedded indices. This does not include "tokens" which
            # is just used to compute the loss in LanguageModel.
            raise ConfigurationError(f"LM from {archive_file} trained with multiple embedders!")
        if "embedder_to_indexer_map" in text_field_embedder:
            # Similarly we don't support multiple indexers per embedder.
            raise ConfigurationError(f"LM from {archive_file} trained with embedder_to_indexer_map!")
        self._token_name = token_names[0]

        # TODO(brendanr): Find a way to remove this hack. The issue fundamentally is that the
        # BasicTextFieldEmbedder concatenates multiple embedded representations. When a
        # downstream model uses both, tokens and token characters, say, and only adds bos/eos
        # tokens to the token characters, the dimensions don't match. See:
        # https://github.com/allenai/allennlp/blob/eff25a3085aa9976a7650d30d8961c3626ddc411/allennlp/modules/text_field_embedders/basic_text_field_embedder.py#L109
        #
        # For the equivalent hack in the ELMo embedder see:
        # https://github.com/allenai/allennlp/blob/eff25a3085aa9976a7650d30d8961c3626ddc411/allennlp/modules/elmo.py#L590
        if bos_eos_tokens:
            dataset_reader_config = config.get("dataset_reader")
            if dataset_reader_config.get("type") == "multiprocess":
                dataset_reader_config = dataset_reader_config.get("base_reader")
            token_indexer_config = dataset_reader_config.get("token_indexers").get(self._token_name)
            token_indexer: TokenIndexer = TokenIndexer.from_params(token_indexer_config)
            token_list = [Token(token) for token in bos_eos_tokens]
            # TODO(brendanr): Obtain these indices from the vocab once the
            # ELMoTokenCharactersIndexer adds the mappings.
            bos_eos_indices = token_indexer.tokens_to_indices(token_list, self._lm.vocab, "key")["key"]
            self._bos_indices = torch.Tensor(bos_eos_indices[0])
            self._eos_indices = torch.Tensor(bos_eos_indices[1])
        else:
            self._bos_indices = None
            self._eos_indices = None

        if dropout:
            self._dropout = torch.nn.Dropout(dropout)
        else:
            self._dropout = lambda x: x

        self._remove_bos_eos = remove_bos_eos
        num_layers = self._lm.num_layers()
        # TODO(brendanr): Consider passing our LM as a custom module to `Elmo` instead.
        # See https://github.com/allenai/allennlp/blob/master/allennlp/modules/elmo.py#L76
        self._scalar_mix = ScalarMix(mixture_size=num_layers, do_layer_norm=False, trainable=True)

        # pylint: disable=protected-access
        character_dim = self._lm._text_field_embedder.get_output_dim()
        contextual_dim = self._lm._contextualizer.get_output_dim()

        if contextual_dim % character_dim != 0:
            raise ConfigurationError(
                    "The output dimensions for the text_field_embedder " +
                    f"({character_dim}) and the contextualizer ({contextual_dim})" +
                    f" from the language model loaded from {archive_file} are " +
                    "not compatible. Please check the config used to train that " +
                    "model and ensure that the output dimension of the " +
                    "text_field_embedder divides the output dimension of the " +
                    "contextualizer.")
        self._character_embedding_duplication_count = contextual_dim // character_dim

        for param in self._lm.parameters():
            param.requires_grad = requires_grad

    def get_output_dim(self) -> int:
        return self._lm._contextualizer.get_output_dim() # pylint: disable=protected-access

    def forward(self,  # type: ignore
                inputs: torch.Tensor) -> Dict[str, torch.Tensor]:
        """
        Parameters
        ----------
        inputs: ``torch.Tensor``
            Shape ``(batch_size, timesteps, ...)`` of token ids representing the current batch.
            These must have been produced using the same indexer the LM was trained on.

        Returns
        -------
        The bidirectional language model representations for the input sequence, shape
        ``(batch_size, timesteps, embedding_dim)``
        """
        # pylint: disable=arguments-differ
        if self._bos_indices is not None:
            mask = get_text_field_mask({"": inputs})
            inputs, mask = add_sentence_boundary_token_ids(
                    inputs, mask, self._bos_indices, self._eos_indices
            )

        source = {self._token_name: inputs}
        result_dict = self._lm(source)

        # shape (batch_size, timesteps, embedding_size)
        noncontextual_token_embeddings = result_dict["noncontextual_token_embeddings"]
        contextual_embeddings = result_dict["lm_embeddings"]

        # Typically the non-contextual embeddings are smaller than the contextualized embeddings.
        # Since we're averaging all the layers we need to make their dimensions match. Simply
        # repeating the non-contextual embeddings is a crude, but effective, way to do this.
        duplicated_character_embeddings = torch.cat(
                [noncontextual_token_embeddings] * self._character_embedding_duplication_count, -1
        )
        averaged_embeddings = self._scalar_mix(
                [duplicated_character_embeddings] + contextual_embeddings
        )

        # Add dropout
        averaged_embeddings = self._dropout(averaged_embeddings)
        if self._remove_bos_eos:
            averaged_embeddings, _ = remove_sentence_boundaries(
                    averaged_embeddings, result_dict["mask"]
            )

        return averaged_embeddings



# pylint: disable=no-self-use,invalid-name,line-too-long,no-member

import json
import os
import sys

from allennlp.common.testing import AllenNlpTestCase
from allennlp.service.config_explorer import make_app


class TestConfigExplorer(AllenNlpTestCase):

    def setUp(self):
        super().setUp()
        app = make_app()
        app.testing = True
        self.client = app.test_client()

    def test_app(self):
        response = self.client.get('/')
        html = response.get_data().decode('utf-8')

        assert "AllenNLP Configuration Wizard" in html

    def test_api(self):
        response = self.client.get('/api/config/')
        data = json.loads(response.get_data())

        assert data["className"] == ""

        items = data["config"]['items']

        assert items[0] == {
                "name": "dataset_reader",
                "configurable": True,
                "registrable": True,
                "comment": "specify your dataset reader here",
                "annotation": {'origin': "allennlp.data.dataset_readers.dataset_reader.DatasetReader"}
        }


    def test_choices(self):
        response = self.client.get('/api/config/?class=allennlp.data.dataset_readers.dataset_reader.DatasetReader&get_choices=true')
        data = json.loads(response.get_data())

        assert "allennlp.data.dataset_readers.reading_comprehension.squad.SquadReader" in data["choices"]

    def test_subclass(self):
        response = self.client.get('/api/config/?class=allennlp.data.dataset_readers.semantic_role_labeling.SrlReader')
        data = json.loads(response.get_data())

        config = data['config']
        items = config['items']
        assert config['type'] == 'srl'
        assert items[0]["name"] == "token_indexers"

    def test_instantiable_registrable(self):
        response = self.client.get('/api/config/?class=allennlp.data.vocabulary.Vocabulary')
        data = json.loads(response.get_data())
        assert 'config' in data
        assert 'choices' not in data

        response = self.client.get('/api/config/?class=allennlp.data.vocabulary.Vocabulary&get_choices=true')
        data = json.loads(response.get_data())
        assert 'config' not in data
        assert 'choices' in data

    def test_get_choices_failover(self):
        """
        Tests that if we try to get_choices on a non-registrable class
        it just fails back to the config.
        """
        response = self.client.get('/api/config/?class=allennlp.modules.feedforward.FeedForward&get_choices=true')
        data = json.loads(response.get_data())
        assert 'config' in data
        assert 'choices' not in data


    def test_torch_class(self):
        response = self.client.get('/api/config/?class=torch.optim.rmsprop.RMSprop')
        data = json.loads(response.get_data())
        config = data['config']
        items = config['items']

        assert config["type"] == "rmsprop"
        assert any(item["name"] == "lr" for item in items)

    def test_rnn_hack(self):
        """
        Behind the scenes, when you try to create a torch RNN,
        it just calls torch.RNNBase with an extra parameter.
        This test is to make sure that works correctly.
        """
        response = self.client.get('/api/config/?class=torch.nn.modules.rnn.LSTM')
        data = json.loads(response.get_data())
        config = data['config']
        items = config['items']

        assert config["type"] == "lstm"
        assert any(item["name"] == "batch_first" for item in items)

    def test_initializers(self):
        response = self.client.get('/api/config/?class=allennlp.nn.initializers.Initializer&get_choices=true')
        data = json.loads(response.get_data())

        assert 'torch.nn.init.constant_' in data["choices"]
        assert 'allennlp.nn.initializers.block_orthogonal' in data["choices"]

        response = self.client.get('/api/config/?class=torch.nn.init.uniform_')
        data = json.loads(response.get_data())
        config = data['config']
        items = config['items']

        assert config["type"] == "uniform"
        assert any(item["name"] == "a" for item in items)

    def test_regularizers(self):
        response = self.client.get('/api/config/?class=allennlp.nn.regularizers.regularizer.Regularizer&get_choices=true')
        data = json.loads(response.get_data())

        assert 'allennlp.nn.regularizers.regularizers.L1Regularizer' in data["choices"]

        response = self.client.get('/api/config/?class=allennlp.nn.regularizers.regularizers.L1Regularizer')
        data = json.loads(response.get_data())
        config = data['config']
        items = config['items']

        assert config["type"] == "l1"
        assert any(item["name"] == "alpha" for item in items)

    def test_other_modules(self):
        # Create a new package in a temporary dir
        packagedir = self.TEST_DIR / 'configexplorer'
        packagedir.mkdir()  # pylint: disable=no-member
        (packagedir / '__init__.py').touch()  # pylint: disable=no-member

        # And add that directory to the path
        sys.path.insert(0, str(self.TEST_DIR))

        # Write out a duplicate predictor there, but registered under a different name.
        from allennlp.predictors import bidaf
        with open(bidaf.__file__) as f:
            code = f.read().replace("""@Predictor.register('machine-comprehension')""",
                                    """@Predictor.register('config-explorer-predictor')""")

        with open(os.path.join(packagedir, 'predictor.py'), 'w') as f:
            f.write(code)

        # Without specifying modules to load, it shouldn't be there
        app = make_app()
        app.testing = True
        client = app.test_client()
        response = client.get('/api/config/?class=allennlp.predictors.predictor.Predictor&get_choices=true')
        data = json.loads(response.get_data())
        assert "allennlp.predictors.bidaf.BidafPredictor" in data["choices"]
        assert "configexplorer.predictor.BidafPredictor" not in data["choices"]

        # With specifying extra modules, it should be there.
        app = make_app(['configexplorer'])
        app.testing = True
        client = app.test_client()
        response = client.get('/api/config/?class=allennlp.predictors.predictor.Predictor&get_choices=true')
        data = json.loads(response.get_data())
        assert "allennlp.predictors.bidaf.BidafPredictor" in data["choices"]
        assert "configexplorer.predictor.BidafPredictor" in data["choices"]

        sys.path.remove(str(self.TEST_DIR))



import atexit
import logging
import os
import pathlib
import shutil
import subprocess
import requests

from allennlp.common.file_utils import cached_path
from allennlp.common.checks import check_for_java

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

SEMPRE_EXECUTOR_JAR = "https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-executor-0.1.0.jar"
ABBREVIATIONS_FILE = "https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-abbreviations.tsv"
GROW_FILE = "https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-grow.grammar"
SEMPRE_DIR = str(pathlib.Path('data/'))
SEMPRE_ABBREVIATIONS_PATH = os.path.join(SEMPRE_DIR, "abbreviations.tsv")
SEMPRE_GRAMMAR_PATH = os.path.join(SEMPRE_DIR, "grow.grammar")


class WikiTablesSempreExecutor:
    """
    This class evaluates lambda-DCS logical forms by calling out to SEMPRE, where the particular
    lambda-DCS language we use was defined.  It's a huge pain to have to rely on a call to a java
    subprocess, but it's way easier than trying to write our own executor for the language.

    Because of how the SEMPRE executor works, we need to have access to the original table on disk,
    and we need to pass in the full lisp "example" string that's given in the dataset.  SEMPRE
    parses the example string (which includes a path to the table), reads the table, executes the
    logical form in the context of the table, and compares the answer produced to the answer
    specified in the example string.

    We don't even get back the denotation of the logical form here, because then we'd have to do a
    comparison with the correct answer, and that's a bit messy - better to just let SEMPRE do it.
    This is why we only provide a :func:`evaluate_logical_form` method that returns a ``bool``
    instead of an ``execute`` method returning an answer.  You might think that if we got the
    answer back, we could at least use this for a demo.  The sad thing is that even that doesn't
    work, because this executor relies on having the table for the example already accessible on
    disk, which we don't have in the case of a demo - we have to do extra stuff there to get it to
    work, including writing the table to disk so that SEMPRE can access it!  It's all a bit of a
    mess.
    """
    def __init__(self, table_directory: str) -> None:
        self._table_directory = table_directory
        self._executor_process: subprocess.Popen = None
        self._should_remove_sempre_dir = not os.path.exists(SEMPRE_DIR)
        self._create_sempre_executor()

    def evaluate_logical_form(self, logical_form: str, example_lisp_string: str) -> bool:
        if not logical_form or logical_form.startswith('Error'):
            return False
        if example_lisp_string[-1] != '\n':
            example_lisp_string += '\n'
        if logical_form[-1] != '\n':
            logical_form += '\n'
        self._executor_process.stdin.write(example_lisp_string.encode('utf-8'))
        self._executor_process.stdin.write(logical_form.encode('utf-8'))
        self._executor_process.stdin.flush()
        result = self._executor_process.stdout.readline().decode().strip()
        return result == '1.0'

    def _create_sempre_executor(self) -> None:
        """
        Creates a server running SEMPRE that we can send logical forms to for evaluation.  This
        uses inter-process communication, because SEMPRE is java code.  We also need to be careful
        to clean up the process when our program exits.
        """
        if self._executor_process:
            return

        # It'd be much nicer to just use `cached_path` for these files.  However, the SEMPRE jar
        # that we're using expects to find these files in a particular location, so we need to make
        # sure we put the files in that location.
        os.makedirs(SEMPRE_DIR, exist_ok=True)
        abbreviations_path = os.path.join(SEMPRE_DIR, 'abbreviations.tsv')
        if not os.path.exists(abbreviations_path):
            result = requests.get(ABBREVIATIONS_FILE)
            with open(abbreviations_path, 'wb') as downloaded_file:
                downloaded_file.write(result.content)

        grammar_path = os.path.join(SEMPRE_DIR, 'grow.grammar')
        if not os.path.exists(grammar_path):
            result = requests.get(GROW_FILE)
            with open(grammar_path, 'wb') as downloaded_file:
                downloaded_file.write(result.content)

        if not check_for_java():
            raise RuntimeError('Java is not installed properly.')
        args = ['java', '-jar', cached_path(SEMPRE_EXECUTOR_JAR), 'serve', self._table_directory]
        self._executor_process = subprocess.Popen(args,
                                                  stdin=subprocess.PIPE,
                                                  stdout=subprocess.PIPE,
                                                  bufsize=1)

        lines = []
        for _ in range(6):
            # SEMPRE outputs six lines of stuff when it loads that I can't disable.  So, we clear
            # that here.
            lines.append(str(self._executor_process.stdout.readline()))
        assert 'Parser' in lines[-1], "SEMPRE server output unexpected; the server may have changed"
        logger.info("Started SEMPRE server for evaluating logical forms")

        # This is supposed to ensure that the subprocess gets killed when python exits.
        atexit.register(self._stop_sempre_executor)

    def _stop_sempre_executor(self) -> None:
        if not self._executor_process:
            return
        self._executor_process.terminate()
        self._executor_process = None
        logger.info("Stopped SEMPRE server")
        if self._should_remove_sempre_dir and os.path.exists(SEMPRE_DIR):
            shutil.rmtree(SEMPRE_DIR)
            logger.info(f"Removed SEMPRE data directory ({SEMPRE_DIR})")



# pylint: disable=no-self-use,invalid-name
import numpy
import pytest

from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data.fields import MultiLabelField
from allennlp.data.vocabulary import Vocabulary


class TestMultiLabelField(AllenNlpTestCase):
    def test_as_tensor_returns_integer_tensor(self):
        f = MultiLabelField([2, 3], skip_indexing=True, label_namespace="test1", num_labels=5)
        tensor = f.as_tensor(f.get_padding_lengths()).detach().cpu().tolist()
        assert tensor == [0, 0, 1, 1, 0]
        assert set([type(item) for item in tensor]) == set([int])

    def test_multilabel_field_can_index_with_vocab(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace("rel0", namespace="rel_labels")
        vocab.add_token_to_namespace("rel1", namespace="rel_labels")
        vocab.add_token_to_namespace("rel2", namespace="rel_labels")

        f = MultiLabelField(["rel1", "rel0"], label_namespace="rel_labels")
        f.index(vocab)
        tensor = f.as_tensor(f.get_padding_lengths()).detach().cpu().numpy()
        numpy.testing.assert_array_almost_equal(tensor, numpy.array([1, 1, 0]))

    def test_multilabel_field_raises_with_non_integer_labels_and_no_indexing(self):
        with pytest.raises(ConfigurationError):
            _ = MultiLabelField(["non integer field"], skip_indexing=True)

    def test_multilabel_field_raises_with_no_indexing_and_missing_num_labels(self):
        with pytest.raises(ConfigurationError):
            _ = MultiLabelField([0, 2], skip_indexing=True, num_labels=None)

    def test_multilabel_field_raises_with_no_indexing_and_wrong_num_labels(self):
        with pytest.raises(ConfigurationError):
            _ = MultiLabelField([0, 2, 4], skip_indexing=True, num_labels=3)

    def test_multilabel_field_raises_with_incorrect_label_type(self):
        with pytest.raises(ConfigurationError):
            _ = MultiLabelField([1, 2], skip_indexing=False)

    def test_multilabel_field_raises_with_given_num_labels(self):
        with pytest.raises(ConfigurationError):
            _ = MultiLabelField([1, 2], skip_indexing=False, num_labels=4)

    def test_multilabel_field_empty_field_works(self):
        vocab = Vocabulary()
        vocab.add_token_to_namespace("label1", namespace="test_empty_labels")
        vocab.add_token_to_namespace("label2", namespace="test_empty_labels")

        f = MultiLabelField([], label_namespace="test_empty_labels")
        f.index(vocab)
        tensor = f.as_tensor(f.get_padding_lengths()).detach().cpu().numpy()
        numpy.testing.assert_array_almost_equal(tensor, numpy.array([0, 0]))

    def test_class_variables_for_namespace_warnings_work_correctly(self):
        # pylint: disable=protected-access
        assert "text" not in MultiLabelField._already_warned_namespaces
        with self.assertLogs(logger="allennlp.data.fields.multilabel_field", level="WARNING"):
            _ = MultiLabelField(["test"], label_namespace="text")

        # We've warned once, so we should have set the class variable to False.
        assert "text" in MultiLabelField._already_warned_namespaces
        with pytest.raises(AssertionError):
            with self.assertLogs(logger="allennlp.data.fields.multilabel_field", level="WARNING"):
                _ = MultiLabelField(["test2"], label_namespace="text")

        # ... but a new namespace should still log a warning.
        assert "text2" not in MultiLabelField._already_warned_namespaces
        with self.assertLogs(logger="allennlp.data.fields.multilabel_field", level="WARNING"):
            _ = MultiLabelField(["test"], label_namespace="text2")

    def test_printing_doesnt_crash(self):
        field = MultiLabelField(["label"], label_namespace="namespace")
        print(field)



import json
import os
import sys
import argparse

sys.path.insert(0, os.path.dirname(os.path.abspath(os.path.join(__file__, os.pardir))))

from allennlp.data.dataset_readers.dataset_utils.text2sql_utils import process_sql_data, SqlData
from allennlp.semparse.contexts.sql_context_utils import SqlVisitor, format_grammar_string
from allennlp.semparse.contexts.text2sql_table_context import GRAMMAR_DICTIONARY
from parsimonious.grammar import Grammar
# still TODO: 
# JOIN, seems hard.
# Added query to pos_value - check this, very unclear if it is the correct way to handle this.
# not all functions can take * as an argument.
# Check whether LIKE can take non string arguments (example in scholar dataset)


def parse_dataset(filename: str, filter_by: str = None, verbose: bool = False):

    grammar_string = format_grammar_string(GRAMMAR_DICTIONARY)
    grammar = Grammar(grammar_string)

    filter_by = filter_by or "13754332dvmklfdsaf-3543543"
    data = json.load(open(filename))
    num_queries = 0
    num_parsed = 0
    filtered_errors = 0

    non_basic_as_aliases = 0
    as_count = 0
    queries_with_weird_as = 0

    for i, sql_data in enumerate(process_sql_data(data)):
        sql_visitor = SqlVisitor(grammar)

        if any([x[:7] == "DERIVED"] for x in sql_data.sql):
            # NOTE: DATA hack alert - the geography dataset doesn't alias derived tables consistently,
            # so we fix the data a bit here instead of completely re-working the grammar.
            sql_to_use = []
            for j, token in enumerate(sql_data.sql):
                if token[:7] == "DERIVED" and sql_data.sql[j-1] == ")":
                    sql_to_use.append("AS")
                sql_to_use.append(token)

            previous_token = None
            query_has_weird_as = False
            for j, token in enumerate(sql_to_use[:-1]):

                if token == "AS" and previous_token is not None:

                    table_name = sql_to_use[j + 1][:-6]
                    if table_name != previous_token:
                        non_basic_as_aliases += 1
                        query_has_weird_as = True
                    as_count += 1
                previous_token = token

            if query_has_weird_as:
                queries_with_weird_as += 1


            sql_string = " ".join(sql_to_use)
        else:
            sql_string = " ".join(sql_data.sql)
        num_queries += 1
        try:
            prod_rules = sql_visitor.parse(sql_string)
            num_parsed += 1
        except Exception as e:

            if filter_by in sql_string:
                filtered_errors += 1

            if verbose and filter_by not in sql_string:
                print()
                print(e)
                print(" ".join(sql_data.text))
                print(sql_data.sql)
                try:
                    import sqlparse
                    print(sqlparse.format(sql_string, reindent=True))
                except Exception:
                    print(sql_string)

        if (i + 1) % 500 == 0:
            print(f"\tProcessed {i + 1} queries.")

    return num_parsed, num_queries, filtered_errors, non_basic_as_aliases, as_count, queries_with_weird_as

def main(data_directory: int, dataset: str = None, filter_by: str = None, verbose: bool = False) -> None:
    """
    Parameters
    ----------
    data_directory : str, required.
        The path to the data directory of https://github.com/jkkummerfeld/text2sql-data
        which has been preprocessed using scripts/reformat_text2sql_data.py.
    dataset : str, optional.
        The dataset to parse. By default all are parsed.
    filter_by : str, optional
        Compute statistics about a particular error and only print errors which don't contain this string.
    verbose : bool, optional.
        Whether to print information about incorrectly parsed SQL.
    """
    directory_dict = {path: files for path, names, files in os.walk(data_directory) if files}

    for directory, data_files in directory_dict.items():
        if "query_split" in directory or  (dataset is not None and dataset not in directory):
            continue

        print(f"Parsing dataset at {directory}")
        parsed = 0
        total_non_aliases = 0
        total_as_count = 0
        total_queries_with_weird_as = 0
        total = 0
        for json_file in data_files:
            print(f"\tParsing split at {json_file}")
            file_path = os.path.join(directory, json_file)
            num_parsed, num_queries, filtered_errors, non_basic_as_aliases, as_count, queries_with_weird_as = parse_dataset(file_path, filter_by, verbose)

            parsed += num_parsed
            total += num_queries
            total_non_aliases += non_basic_as_aliases
            total_as_count += as_count
            total_queries_with_weird_as += queries_with_weird_as

        print(f"\tParsed {parsed} out of {total} queries, coverage {parsed/total}")
        print(f"\tFound {total_non_aliases} out of {total_as_count} non simple AS aliases. percentage: {total_non_aliases/total_as_count}")
        print(f"\tFound {total_queries_with_weird_as} out of {total} queries with > 1 weird AS. percentage: {total_queries_with_weird_as/total}")
        if filter_by is not None:
            print(f"\tOf {total - parsed} errors, {filtered_errors/ (total - parsed + 1e-13)} contain {filter_by}")

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description="Check the coverage of a SQL Grammar on the text2sql datasets.")
    parser.add_argument('--data', type=str, help='The path to the text2sql data directory.')
    parser.add_argument('--dataset', type=str, default=None,
                        help='The dataset to check coverage for. Defaults to all datasets.')
    parser.add_argument('--filter', type=str, default=None, help='A string to filter by.')
    parser.add_argument('--verbose', help='Verbose output.', action='store_true')
    args = parser.parse_args()
    main(args.data, args.dataset, args.filter, args.verbose)



# pylint: disable=no-self-use,invalid-name,not-callable
import numpy
import pytest
import torch

from allennlp.modules import Pruner
from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.util import batched_index_select

class TestPruner(AllenNlpTestCase):
    def test_pruner_selects_top_scored_items_and_respects_masking(self):
        # Really simple scorer - sum up the embedding_dim.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = Pruner(scorer=scorer)

        items = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        items[0, :2, :] = 1
        items[1, 2:, :] = 1
        items[2, 2:, :] = 1

        mask = torch.ones([3, 4])
        mask[1, 0] = 0
        mask[1, 3] = 0
        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(items, mask, 2)

        # Second element in the batch would have indices 2, 3, but
        # 3 and 0 are masked, so instead it has 1, 2.
        numpy.testing.assert_array_equal(pruned_indices.data.numpy(), numpy.array([[0, 1],
                                                                                   [1, 2],
                                                                                   [2, 3]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.ones([3, 2]))

        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(items, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements.
        numpy.testing.assert_array_equal(correct_embeddings.sum(-1).unsqueeze(-1).data.numpy(),
                                         pruned_scores.data.numpy())

    def test_scorer_raises_with_incorrect_scorer_spec(self):
        # Mis-configured scorer - doesn't produce a tensor with 1 as it's final dimension.
        scorer = lambda tensor: tensor.sum(-1)
        pruner = Pruner(scorer=scorer) # type: ignore
        items = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        mask = torch.ones([3, 4])

        with pytest.raises(ValueError):
            _ = pruner(items, mask, 2)

    def test_scorer_works_for_completely_masked_rows(self):
        # Really simple scorer - sum up the embedding_dim.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = Pruner(scorer=scorer) # type: ignore

        items = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        items[0, :2, :] = 1
        items[1, 2:, :] = 1
        items[2, 2:, :] = 1

        mask = torch.ones([3, 4])
        mask[1, 0] = 0
        mask[1, 3] = 0
        mask[2, :] = 0 # fully masked last batch element.

        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(items, mask, 2)

        # We can't check the last row here, because it's completely masked.
        # Instead we'll check that the scores for these elements are very small.
        numpy.testing.assert_array_equal(pruned_indices[:2].data.numpy(), numpy.array([[0, 1],
                                                                                       [1, 2]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.array([[1, 1],
                                                                                [1, 1],
                                                                                [0, 0]]))
        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(items, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements, with masked elements very
        # small (but not -inf, because that can cause problems).  We'll test these two cases
        # separately.
        correct_scores = correct_embeddings.sum(-1).unsqueeze(-1).data.numpy()
        numpy.testing.assert_array_equal(correct_scores[:2], pruned_scores[:2].data.numpy())
        numpy.testing.assert_array_equal(pruned_scores[2] < -1e15, [[1], [1]])
        numpy.testing.assert_array_equal(pruned_scores[2] == float('-inf'), [[0], [0]])

    def test_pruner_selects_top_scored_items_and_respects_masking_different_num_items(self):
        # Really simple scorer - sum up the embedding_dim.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = Pruner(scorer=scorer)

        items = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        items[0, 0, :] = 1.5
        items[0, 1, :] = 2
        items[0, 3, :] = 1
        items[1, 1:3, :] = 1
        items[2, 0, :] = 1
        items[2, 1, :] = 2
        items[2, 2, :] = 1.5

        mask = torch.ones([3, 4])
        mask[1, 3] = 0

        num_items_to_keep = torch.tensor([3, 2, 1], dtype=torch.long)

        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(
                items, mask, num_items_to_keep)

        # Second element in the batch would have indices 2, 3, but
        # 3 and 0 are masked, so instead it has 1, 2.
        numpy.testing.assert_array_equal(pruned_indices.data.numpy(), numpy.array([[0, 1, 3],
                                                                                   [1, 2, 2],
                                                                                   [1, 2, 2]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.array([[1, 1, 1],
                                                                                [1, 1, 0],
                                                                                [1, 0, 0]]))

        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(items, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements.
        numpy.testing.assert_array_equal(correct_embeddings.sum(-1).unsqueeze(-1).data.numpy(),
                                         pruned_scores.data.numpy())

    def test_pruner_works_for_row_with_no_items_requested(self):
        # Case where `num_items_to_keep` is a tensor rather than an int. Make sure it does the right
        # thing when no items are requested for one of the rows.
        scorer = lambda tensor: tensor.sum(-1).unsqueeze(-1)
        pruner = Pruner(scorer=scorer)

        items = torch.randn([3, 4, 5]).clamp(min=0.0, max=1.0)
        items[0, :3, :] = 1
        items[1, 2:, :] = 1
        items[2, 2:, :] = 1

        mask = torch.ones([3, 4])
        mask[1, 0] = 0
        mask[1, 3] = 0

        num_items_to_keep = torch.tensor([3, 2, 0], dtype=torch.long)

        pruned_embeddings, pruned_mask, pruned_indices, pruned_scores = pruner(
                items, mask, num_items_to_keep)

        # First element just picks top three entries. Second would pick entries 2 and 3, but 0 and 3
        # are masked, so it takes 1 and 2 (repeating the second index). The third element is
        # entirely masked and just repeats the largest index with a top-3 score.
        numpy.testing.assert_array_equal(pruned_indices.data.numpy(), numpy.array([[0, 1, 2],
                                                                                   [1, 2, 2],
                                                                                   [3, 3, 3]]))
        numpy.testing.assert_array_equal(pruned_mask.data.numpy(), numpy.array([[1, 1, 1],
                                                                                [1, 1, 0],
                                                                                [0, 0, 0]]))

        # embeddings should be the result of index_selecting the pruned_indices.
        correct_embeddings = batched_index_select(items, pruned_indices)
        numpy.testing.assert_array_equal(correct_embeddings.data.numpy(),
                                         pruned_embeddings.data.numpy())
        # scores should be the sum of the correct embedding elements.
        numpy.testing.assert_array_equal(correct_embeddings.sum(-1).unsqueeze(-1).data.numpy(),
                                         pruned_scores.data.numpy())



# pylint: disable=invalid-name,no-self-use
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.modules.seq2seq_encoders.bidirectional_language_model_transformer import \
        BidirectionalLanguageModelTransformer

class TestBidirectionalLanguageModelTransformer(AllenNlpTestCase):
    def test_bidirectional_transformer_encoder(self):
        transformer_encoder = BidirectionalLanguageModelTransformer(input_dim=32,
                                                                    hidden_dim=64,
                                                                    num_layers=2)
        token_embeddings = torch.rand(5, 10, 32)
        mask = torch.ones(5, 10)
        mask[0, 7:] = 0
        mask[1, 5:] = 0

        output = transformer_encoder(token_embeddings, mask)
        assert list(output.size()) == [5, 10, 64]

    def test_bidirectional_transfomer_all_layers(self):
        transformer_encoder = BidirectionalLanguageModelTransformer(input_dim=32,
                                                                    hidden_dim=64,
                                                                    num_layers=2,
                                                                    return_all_layers=True)
        token_embeddings = torch.rand(5, 10, 32)
        mask = torch.ones(5, 10)
        mask[0, 7:] = 0
        mask[1, 5:] = 0

        output = transformer_encoder(token_embeddings, mask)
        assert len(output) == 2

        concat_layers = torch.cat(
                [layer.unsqueeze(1) for layer in output], dim=1
        )

        # (batch_size, num_layers, timesteps, 2*input_dim)
        assert list(concat_layers.size()) == [5, 2, 10, 64]

    def test_attention_masks(self):
        transformer_encoder = BidirectionalLanguageModelTransformer(input_dim=32,
                                                                    hidden_dim=64,
                                                                    num_layers=2)

        mask = torch.ones(3, 6).int()
        mask[0, 3:] = 0
        mask[1, 5:] = 0

        forward_mask, backward_mask = transformer_encoder.get_attention_masks(mask)

        # rows = position in sequence
        # columns = positions used for attention
        assert (forward_mask[0].data == torch.IntTensor([[1, 0, 0, 0, 0, 0],
                                                         [1, 1, 0, 0, 0, 0],
                                                         [1, 1, 1, 0, 0, 0],
                                                         [0, 0, 0, 0, 0, 0],
                                                         [0, 0, 0, 0, 0, 0],
                                                         [0, 0, 0, 0, 0, 0]])).all()

        assert (forward_mask[1].data == torch.IntTensor([[1, 0, 0, 0, 0, 0],
                                                         [1, 1, 0, 0, 0, 0],
                                                         [1, 1, 1, 0, 0, 0],
                                                         [1, 1, 1, 1, 0, 0],
                                                         [1, 1, 1, 1, 1, 0],
                                                         [0, 0, 0, 0, 0, 0]])).all()

        assert (forward_mask[2].data == torch.IntTensor([[1, 0, 0, 0, 0, 0],
                                                         [1, 1, 0, 0, 0, 0],
                                                         [1, 1, 1, 0, 0, 0],
                                                         [1, 1, 1, 1, 0, 0],
                                                         [1, 1, 1, 1, 1, 0],
                                                         [1, 1, 1, 1, 1, 1]])).all()

        assert (backward_mask[0].data == torch.IntTensor([[1, 1, 1, 0, 0, 0],
                                                          [0, 1, 1, 0, 0, 0],
                                                          [0, 0, 1, 0, 0, 0],
                                                          [0, 0, 0, 0, 0, 0],
                                                          [0, 0, 0, 0, 0, 0],
                                                          [0, 0, 0, 0, 0, 0]])).all()

        assert (backward_mask[1].data == torch.IntTensor([[1, 1, 1, 1, 1, 0],
                                                          [0, 1, 1, 1, 1, 0],
                                                          [0, 0, 1, 1, 1, 0],
                                                          [0, 0, 0, 1, 1, 0],
                                                          [0, 0, 0, 0, 1, 0],
                                                          [0, 0, 0, 0, 0, 0]])).all()

        assert (backward_mask[2].data == torch.IntTensor([[1, 1, 1, 1, 1, 1],
                                                          [0, 1, 1, 1, 1, 1],
                                                          [0, 0, 1, 1, 1, 1],
                                                          [0, 0, 0, 1, 1, 1],
                                                          [0, 0, 0, 0, 1, 1],
                                                          [0, 0, 0, 0, 0, 1]])).all()



# pylint: disable=protected-access,not-callable

from collections import Counter
import math

import numpy as np
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.training.metrics import BLEU


class BleuTest(AllenNlpTestCase):

    def setUp(self):
        super().setUp()
        self.metric = BLEU(ngram_weights=(0.5, 0.5), exclude_indices={0})

    def test_get_valid_tokens_mask(self):
        tensor = torch.tensor([[1, 2, 3, 0],
                               [0, 1, 1, 0]])
        result = self.metric._get_valid_tokens_mask(tensor)
        result = result.long().numpy()
        check = np.array([[1, 1, 1, 0],
                          [0, 1, 1, 0]])
        np.testing.assert_array_equal(result, check)

    def test_ngrams(self):
        tensor = torch.tensor([1, 2, 3, 1, 2, 0])

        # Unigrams.
        counts = Counter(self.metric._ngrams(tensor, 1))
        unigram_check = {(1,): 2, (2,): 2, (3,): 1}
        assert counts == unigram_check

        # Bigrams.
        counts = Counter(self.metric._ngrams(tensor, 2))
        bigram_check = {(1, 2): 2, (2, 3): 1, (3, 1): 1}
        assert counts == bigram_check

        # Trigrams.
        counts = Counter(self.metric._ngrams(tensor, 3))
        trigram_check = {(1, 2, 3): 1, (2, 3, 1): 1, (3, 1, 2): 1}
        assert counts == trigram_check

        # ngram size too big, no ngrams produced.
        counts = Counter(self.metric._ngrams(tensor, 7))
        assert counts == {}

    def test_bleu_computed_correctly(self):
        self.metric.reset()

        # shape: (batch_size, max_sequence_length)
        predictions = torch.tensor([[1, 0, 0],
                                    [1, 1, 0],
                                    [1, 1, 1]])

        # shape: (batch_size, max_gold_sequence_length)
        gold_targets = torch.tensor([[2, 0, 0],
                                     [1, 0, 0],
                                     [1, 1, 2]])

        self.metric(predictions, gold_targets)

        assert self.metric._prediction_lengths == 6
        assert self.metric._reference_lengths == 5

        # Number of unigrams in predicted sentences that match gold sentences
        # (but not more than maximum occurence of gold unigram within batch).
        assert self.metric._precision_matches[1] == (
                0 +  # no matches in first sentence.
                1 +  # one clipped match in second sentence.
                2    # two clipped matches in third sentence.
        )

        # Total number of predicted unigrams.
        assert self.metric._precision_totals[1] == (
                1 +
                2 +
                3
        )

        # Number of bigrams in predicted sentences that match gold sentences
        # (but not more than maximum occurence of gold bigram within batch).
        assert self.metric._precision_matches[2] == (
                0 +
                0 +
                1
        )

        # Total number of predicted bigrams.
        assert self.metric._precision_totals[2] == (
                0 +
                1 +
                2
        )

        # Brevity penalty should be 1.0
        assert self.metric._get_brevity_penalty() == 1.0

        bleu = self.metric.get_metric(reset=True)["BLEU"]
        check = math.exp(0.5 * (math.log(3) - math.log(6)) +
                         0.5 * (math.log(1) - math.log(3)))
        np.testing.assert_approx_equal(bleu, check)

    def test_bleu_computed_with_zero_counts(self):
        self.metric.reset()
        assert self.metric.get_metric()["BLEU"] == 0



#! /usr/bin/env python
# pylint: disable=invalid-name,redefined-outer-name
from glob import glob
import os
import re
import sys
from typing import Set

DOCS_DIR = 'doc/api'
MODULE_REGEX = r"\ballennlp\.[a-z0-9_.]+\b"
MODULE_GLOB = 'allennlp/**/*.py'

MODULES_THAT_NEED_NO_DOCS: Set[str] = {
        # No docs at top level.
        'allennlp',
        'allennlp.version',
        # No docs for tests, unnecessary info for users.
        'allennlp.tests.*',
        # Private base class, no docs needed.
        'allennlp.modules.encoder_base',
        # Deprecated in favor of allennlp.models.language_model
        'allennlp.models.bidirectional_lm',
        # Deprecated in favor of
        # allennlp.modules.token_embedders.language_model_token_embedder
        'allennlp.modules.token_embedders.bidirectional_language_model_token_embedder',
        # Deprecated in favor of allennlp.predictors.
        'allennlp.service.predictors.*',
}

DOCS_THAT_NEED_NO_MODULES: Set[str] = {
        # Function is defined in allennlp/commands/__init__.py.
        'allennlp.commands.main',
}


def documented_modules(docs_dir: str = DOCS_DIR, module_regex: str = MODULE_REGEX) -> Set[str]:
    modules: Set[str] = set()

    for path in glob(os.path.join(docs_dir, '**/*.rst'), recursive=True):
        with open(path) as f:
            text = f.read()
            for module in re.findall(module_regex, text):
                modules.add(module)

    return modules


def existing_modules(module_glob: str = MODULE_GLOB) -> Set[str]:
    modules: Set[str] = set()

    for path in glob(module_glob, recursive=True):
        path = re.sub(".py$", "", path)
        path = re.sub("/__init__", "", path)
        path = path.replace("/", ".")
        modules.add(path)

    return modules

if __name__ == "__main__":
    success = True
    existing = existing_modules()
    documented = documented_modules()
    for module in sorted(existing):
        if module not in documented and not any(re.fullmatch(ignored_module, module) for ignored_module
                                                in MODULES_THAT_NEED_NO_DOCS):
            print("undocumented module:", module)
            success = False
    for module in sorted(documented):
        if module not in existing and module not in DOCS_THAT_NEED_NO_MODULES:
            print("documented but nonexistent:", module)
            success = False

    if not success:
        print("To pass this check, you must add a reference to undocumented modules in our API "
              "documentation in the appropriate place under doc/api (typically, these are modules "
              "that you added in this PR), and remove any reference to modules that you deleted.")
        sys.exit(1)



from allennlp.common.testing import AllenNlpTestCase

from allennlp.semparse.domain_languages import QuaRelLanguage
from allennlp.tests.semparse.domain_languages.domain_language_test import check_productions_match


class QuaRelLanguageTest(AllenNlpTestCase):
    def setUp(self):
        super().setUp()
        self.language = QuaRelLanguage()

    def test_infer_quarel(self):
        assert self.language.execute(('(infer (speed higher world1) (friction higher world1) '
                                      '(friction lower world1))')) == 1

        assert self.language.execute(('(infer (speed higher world2) (friction higher world1) '
                                      '(friction lower world1))')) == 0

        # Both answer options are correct.
        assert self.language.execute(('(infer (speed higher world2) (friction higher world1) '
                                      '(friction higher world1))')) == -2

        # Neither answer option is correct.
        assert self.language.execute(('(infer (speed higher world2) (friction higher world2) '
                                      '(friction higher world2))')) == -1

        assert self.language.logical_form_to_action_sequence(('(infer (speed higher world1) '
                                                              '(friction higher world1) '
                                                              '(friction lower world1))')) == \
                ['@start@ -> int',
                 'int -> [<QuaRelType,QuaRelType,QuaRelType:int>, QuaRelType, QuaRelType, QuaRelType]',
                 '<QuaRelType,QuaRelType,QuaRelType:int> -> infer',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> speed',
                 'Direction -> higher',
                 'World -> world1',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> friction',
                 'Direction -> higher',
                 'World -> world1',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> friction',
                 'Direction -> lower',
                 'World -> world1']

    def test_infer_quaval(self):
        assert self.language.execute(('(infer (and (thickness low world1) '
                                      '(thickness high world2)) '
                                      '(strength lower world1) '
                                      '(strength lower world2))')) == 0

        assert self.language.execute(('(infer (and (thickness low world1) '
                                      '(thickness high world2)) '
                                      '(strength lower world2) '
                                      '(strength lower world1))')) == 1

        assert self.language.logical_form_to_action_sequence(('(infer (and (thickness low world1) '
                                                              '(thickness high world2)) '
                                                              '(strength lower world1) '
                                                              '(strength lower world2))')) == \
                ['@start@ -> int',
                 'int -> [<QuaRelType,QuaRelType,QuaRelType:int>, QuaRelType, QuaRelType, '
                 'QuaRelType]',
                 '<QuaRelType,QuaRelType,QuaRelType:int> -> infer',
                 'QuaRelType -> [<QuaRelType,QuaRelType:QuaRelType>, QuaRelType, QuaRelType]',
                 '<QuaRelType,QuaRelType:QuaRelType> -> and',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> thickness',
                 'Direction -> low',
                 'World -> world1',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> thickness',
                 'Direction -> high',
                 'World -> world2',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> strength',
                 'Direction -> lower',
                 'World -> world1',
                 'QuaRelType -> [<Direction,World:QuaRelType>, Direction, World]',
                 '<Direction,World:QuaRelType> -> strength',
                 'Direction -> lower',
                 'World -> world2']

    def test_action_sequence_to_logical_form(self):
        logical_form = ('(infer (and (thickness low world1) '
                        '(thickness high world2)) '
                        '(strength lower world1) '
                        '(strength lower world2))')
        action_sequence = self.language.logical_form_to_action_sequence(logical_form)
        recovered_logical_form = self.language.action_sequence_to_logical_form(action_sequence)
        assert recovered_logical_form == logical_form

    def test_and_incompatible_setup(self):
        logical_form = ('(infer (and (thickness lower world1) '
                        '(thickness lower world2)) '
                        '(strength lower world1) '
                        '(strength lower world2))')
        assert self.language.execute(logical_form) == -1

    def test_get_nonterminal_productions(self):
        productions = self.language.get_nonterminal_productions()
        assert set(productions.keys()) == {
                'QuaRelType',
                '@start@',
                'World',
                'Direction',
                '<QuaRelType,QuaRelType:QuaRelType>',
                'int',
                '<QuaRelType,QuaRelType,QuaRelType:int>',
                '<Direction,World:QuaRelType>'}
        check_productions_match(productions['QuaRelType'],
                                ['[<QuaRelType,QuaRelType:QuaRelType>, QuaRelType, QuaRelType]',
                                 '[<Direction,World:QuaRelType>, Direction, World]'])
        check_productions_match(productions['@start@'],
                                ['int'])
        check_productions_match(productions['World'],
                                ['world1', 'world2'])
        check_productions_match(productions['Direction'],
                                ['higher', 'lower',
                                 'high', 'low'])
        check_productions_match(productions['<QuaRelType,QuaRelType:QuaRelType>'], ['and'])
        check_productions_match(productions['int'],
                                ['[<QuaRelType,QuaRelType,QuaRelType:int>, QuaRelType, QuaRelType, QuaRelType]'])
        check_productions_match(productions['<QuaRelType,QuaRelType,QuaRelType:int>'], ['infer'])
        check_productions_match(productions['<Direction,World:QuaRelType>'],
                                ["friction", "speed", "distance", "heat", "smoothness", "acceleration",
                                 "amountSweat", "apparentSize", "breakability", "brightness", "exerciseIntensity",
                                 "flexibility", "gravity", "loudness", "mass", "strength", "thickness",
                                 "time", "weight"])



"""
This module contains code for using state machines in a model to do transition-based decoding.
"Transition-based decoding" is where you start in some state, iteratively transition between
states, and have some kind of supervision signal that tells you which end states, or which
transition sequences, are "good".

Typical seq2seq decoding, where you have a fixed vocabulary and no constraints on your output, can
be done much more efficiently than we do in this code.  This is intended for structured models that
have constraints on their outputs.

The key abstractions in this code are the following:

    - ``State`` represents the current state of decoding, containing a list of all of the actions
      taken so far, and a current score for the state.  It also has methods around determining
      whether the state is "finished" and for combining states for batched computation.
    - ``TransitionFunction`` is a ``torch.nn.Module`` that models the transition function between
      states.  Its main method is ``take_step``, which generates a ranked list of next states given
      a current state.
    - ``DecoderTrainer`` is an algorithm for training the transition function with some kind of
      supervision signal.  There are many options for training algorithms and supervision signals;
      this is an abstract class that is generic over the type of the supervision signal.

There is also a generic ``BeamSearch`` class for finding the ``k`` highest-scoring transition
sequences given a trained ``TransitionFunction`` and an initial ``State``.
"""
from allennlp.state_machines.beam_search import BeamSearch
from allennlp.state_machines.constrained_beam_search import ConstrainedBeamSearch
from allennlp.state_machines.states import State
from allennlp.state_machines.trainers import DecoderTrainer
from allennlp.state_machines.transition_functions import TransitionFunction



from collections import defaultdict
from typing import Callable, Dict, List, NamedTuple, Set

from allennlp.common.util import JsonDict
from allennlp.semparse.domain_languages.domain_language import DomainLanguage, predicate


class Object:
    """
    ``Objects`` are the geometric shapes in the NLVR domain. They have values for attributes shape,
    color, x_loc, y_loc and size. We take a dict read from the JSON file and store it here, and
    define a get method for getting the attribute values. We need this to be hashable because need
    to make sets of ``Objects`` during execution, which get passed around between functions.

    Parameters
    ----------
    attributes : ``JsonDict``
        The dict for each object from the json file.
    """
    def __init__(self, attributes: JsonDict, box_id: str) -> None:
        object_color = attributes["color"].lower()
        # The dataset has a hex code only for blue for some reason.
        if object_color.startswith("#"):
            self.color = "blue"
        else:
            self.color = object_color
        object_shape = attributes["type"].lower()
        self.shape = object_shape
        self.x_loc = attributes["x_loc"]
        self.y_loc = attributes["y_loc"]
        self.size = attributes["size"]
        self._box_id = box_id

    def __str__(self):
        if self.size == 10:
            size = "small"
        elif self.size == 20:
            size = "medium"
        else:
            size = "big"
        return f"{size} {self.color} {self.shape} at ({self.x_loc}, {self.y_loc}) in {self._box_id}"

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        return str(self) == str(other)


class Box:
    """
    This class represents each box containing objects in NLVR.

    Parameters
    ----------
    objects_list : ``List[JsonDict]``
        List of objects in the box, as given by the json file.
    box_id : ``int``
        An integer identifying the box index (0, 1 or 2).
    """
    def __init__(self,
                 objects_list: List[JsonDict],
                 box_id: int) -> None:
        self._name = f"box {box_id + 1}"
        self._objects_string = str([str(_object) for _object in objects_list])
        self.objects = set([Object(object_dict, self._name) for object_dict in objects_list])
        self.colors = set([obj.color for obj in self.objects])
        self.shapes = set([obj.shape for obj in self.objects])

    def __str__(self):
        return self._objects_string

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        return str(self) == str(other)


class Color(NamedTuple):
    color: str


class Shape(NamedTuple):
    shape: str


class NlvrLanguage(DomainLanguage):
    # pylint: disable=no-self-use,too-many-public-methods
    def __init__(self, boxes: Set[Box]) -> None:
        self.boxes = boxes
        self.objects: Set[Object] = set()
        for box in self.boxes:
            self.objects.update(box.objects)
        allowed_constants = {
                'color_blue': Color('blue'),
                'color_black': Color('black'),
                'color_yellow': Color('yellow'),
                'shape_triangle': Shape('triangle'),
                'shape_square': Shape('square'),
                'shape_circle': Shape('circle'),
                '1': 1,
                '2': 2,
                '3': 3,
                '4': 4,
                '5': 5,
                '6': 6,
                '7': 7,
                '8': 8,
                '9': 9,
                }
        super().__init__(start_types={bool}, allowed_constants=allowed_constants)

        # Mapping from terminal strings to productions that produce them.
        # Eg.: "yellow" -> "<Set[Object]:Set[Object]> -> yellow"
        # We use this in the agenda-related methods, and some models that use this language look at
        # this field to know how many terminals to plan for.
        self.terminal_productions: Dict[str, str] = {}
        for name, types in self._function_types.items():
            self.terminal_productions[name] = f"{types[0]} -> {name}"

    # These first two methods are about getting an "agenda", which, given an input utterance,
    # tries to guess what production rules should be needed in the logical form.

    def get_agenda_for_sentence(self, sentence: str) -> List[str]:
        """
        Given a ``sentence``, returns a list of actions the sentence triggers as an ``agenda``. The
        ``agenda`` can be used while by a parser to guide the decoder.  sequences as possible. This
        is a simplistic mapping at this point, and can be expanded.

        Parameters
        ----------
        sentence : ``str``
            The sentence for which an agenda will be produced.
        """
        agenda = []
        sentence = sentence.lower()
        if sentence.startswith("there is a box") or sentence.startswith("there is a tower "):
            agenda.append(self.terminal_productions["box_exists"])
        elif sentence.startswith("there is a "):
            agenda.append(self.terminal_productions["object_exists"])

        if "<Set[Box]:bool> -> box_exists" not in agenda:
            # These are object filters and do not apply if we have a box_exists at the top.
            if "touch" in sentence:
                if "top" in sentence:
                    agenda.append(self.terminal_productions["touch_top"])
                elif "bottom" in sentence or "base" in sentence:
                    agenda.append(self.terminal_productions["touch_bottom"])
                elif "corner" in sentence:
                    agenda.append(self.terminal_productions["touch_corner"])
                elif "right" in sentence:
                    agenda.append(self.terminal_productions["touch_right"])
                elif "left" in sentence:
                    agenda.append(self.terminal_productions["touch_left"])
                elif "wall" in sentence or "edge" in sentence:
                    agenda.append(self.terminal_productions["touch_wall"])
                else:
                    agenda.append(self.terminal_productions["touch_object"])
            else:
                # The words "top" and "bottom" may be referring to top and bottom blocks in a tower.
                if "top" in sentence:
                    agenda.append(self.terminal_productions["top"])
                elif "bottom" in sentence or "base" in sentence:
                    agenda.append(self.terminal_productions["bottom"])

            if " not " in sentence:
                agenda.append(self.terminal_productions["negate_filter"])

        if " contains " in sentence or " has " in sentence:
            agenda.append(self.terminal_productions["all_boxes"])
        # This takes care of shapes, colors, top, bottom, big, small etc.
        for constant, production in self.terminal_productions.items():
            # TODO(pradeep): Deal with constant names with underscores.
            if "top" in constant or "bottom" in constant:
                # We already dealt with top, bottom, touch_top and touch_bottom above.
                continue
            if constant in sentence:
                if "<Set[Object]:Set[Object]> ->" in production and "<Set[Box]:bool> -> box_exists" in agenda:
                    if constant in ["square", "circle", "triangle"]:
                        agenda.append(self.terminal_productions[f"shape_{constant}"])
                    elif constant in ["yellow", "blue", "black"]:
                        agenda.append(self.terminal_productions[f"color_{constant}"])
                    else:
                        continue
                else:
                    agenda.append(production)
        # TODO (pradeep): Rules for "member_*" productions ("tower" or "box" followed by a color,
        # shape or number...)
        number_productions = self._get_number_productions(sentence)
        for production in number_productions:
            agenda.append(production)
        if not agenda:
            # None of the rules above was triggered!
            if "box" in sentence:
                agenda.append(self.terminal_productions["all_boxes"])
            else:
                agenda.append(self.terminal_productions["all_objects"])
        return agenda

    @staticmethod
    def _get_number_productions(sentence: str) -> List[str]:
        """
        Gathers all the numbers in the sentence, and returns productions that lead to them.
        """
        # The mapping here is very simple and limited, which also shouldn't be a problem
        # because numbers seem to be represented fairly regularly.
        number_strings = {"one": "1", "two": "2", "three": "3", "four": "4", "five": "5", "six":
                          "6", "seven": "7", "eight": "8", "nine": "9", "ten": "10"}
        number_productions = []
        tokens = sentence.split()
        numbers = number_strings.values()
        for token in tokens:
            if token in numbers:
                number_productions.append(f"int -> {token}")
            elif token in number_strings:
                number_productions.append(f"int -> {number_strings[token]}")
        return number_productions

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return self.boxes == other.boxes and self.objects == other.objects
        return NotImplemented

    # All methods below here are predicates in the NLVR language, or helper methods for them.

    @predicate
    def all_boxes(self) -> Set[Box]:
        return self.boxes

    @predicate
    def all_objects(self) -> Set[Object]:
        return self.objects

    @predicate
    def box_exists(self, boxes: Set[Box]) -> bool:
        return len(boxes) > 0

    @predicate
    def object_exists(self, objects: Set[Object]) -> bool:
        return len(objects) > 0

    @predicate
    def object_in_box(self, box: Set[Box]) -> Set[Object]:
        return_set: Set[Object] = set()
        for box_ in box:
            return_set.update(box_.objects)
        return return_set

    @predicate
    def black(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.color == "black"])

    @predicate
    def blue(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.color == "blue"])

    @predicate
    def yellow(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.color == "yellow"])

    @predicate
    def circle(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.shape == "circle"])

    @predicate
    def square(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.shape == "square"])

    @predicate
    def triangle(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.shape == "triangle"])

    @predicate
    def same_color(self, objects: Set[Object]) -> Set[Object]:
        """
        Filters the set of objects, and returns those objects whose color is the most frequent
        color in the initial set of objects, if the highest frequency is greater than 1, or an
        empty set otherwise.

        This is an unusual name for what the method does, but just as ``blue`` filters objects to
        those that are blue, this filters objects to those that are of the same color.
        """
        return self._get_objects_with_same_attribute(objects, lambda x: x.color)

    @predicate
    def same_shape(self, objects: Set[Object]) -> Set[Object]:
        """
        Filters the set of objects, and returns those objects whose color is the most frequent
        color in the initial set of objects, if the highest frequency is greater than 1, or an
        empty set otherwise.

        This is an unusual name for what the method does, but just as ``triangle`` filters objects
        to those that are triangles, this filters objects to those that are of the same shape.
        """
        return self._get_objects_with_same_attribute(objects, lambda x: x.shape)

    @predicate
    def touch_bottom(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.y_loc + obj.size == 100])

    @predicate
    def touch_left(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.x_loc == 0])

    @predicate
    def touch_top(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.y_loc == 0])

    @predicate
    def touch_right(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.x_loc + obj.size == 100])

    @predicate
    def touch_wall(self, objects: Set[Object]) -> Set[Object]:
        return_set: Set[Object] = set()
        return return_set.union(self.touch_top(objects), self.touch_left(objects),
                                self.touch_right(objects), self.touch_bottom(objects))

    @predicate
    def touch_corner(self, objects: Set[Object]) -> Set[Object]:
        return_set: Set[Object] = set()
        return return_set.union(self.touch_top(objects).intersection(self.touch_right(objects)),
                                self.touch_top(objects).intersection(self.touch_left(objects)),
                                self.touch_bottom(objects).intersection(self.touch_right(objects)),
                                self.touch_bottom(objects).intersection(self.touch_left(objects)))

    @predicate
    def touch_object(self, objects: Set[Object]) -> Set[Object]:
        """
        Returns all objects that touch the given set of objects.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box, box_objects in objects_per_box.items():
            candidate_objects = box.objects
            for object_ in box_objects:
                for candidate_object in candidate_objects:
                    if self._objects_touch_each_other(object_, candidate_object):
                        return_set.add(candidate_object)
        return return_set

    @predicate
    def top(self, objects: Set[Object]) -> Set[Object]:
        """
        Return the topmost objects (i.e. minimum y_loc). The comparison is done separately for each
        box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set: Set[Object] = set()
        for _, box_objects in objects_per_box.items():
            min_y_loc = min([obj.y_loc for obj in box_objects])
            return_set.update(set([obj for obj in box_objects if obj.y_loc == min_y_loc]))
        return return_set

    @predicate
    def bottom(self, objects: Set[Object]) -> Set[Object]:
        """
        Return the bottom most objects(i.e. maximum y_loc). The comparison is done separately for
        each box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set: Set[Object] = set()
        for _, box_objects in objects_per_box.items():
            max_y_loc = max([obj.y_loc for obj in box_objects])
            return_set.update(set([obj for obj in box_objects if obj.y_loc == max_y_loc]))
        return return_set

    @predicate
    def above(self, objects: Set[Object]) -> Set[Object]:
        """
        Returns the set of objects in the same boxes that are above the given objects. That is, if
        the input is a set of two objects, one in each box, we will return a union of the objects
        above the first object in the first box, and those above the second object in the second box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box in objects_per_box:
            # min_y_loc corresponds to the top-most object.
            min_y_loc = min([obj.y_loc for obj in objects_per_box[box]])
            for candidate_obj in box.objects:
                if candidate_obj.y_loc < min_y_loc:
                    return_set.add(candidate_obj)
        return return_set

    @predicate
    def below(self, objects: Set[Object]) -> Set[Object]:
        """
        Returns the set of objects in the same boxes that are below the given objects. That is, if
        the input is a set of two objects, one in each box, we will return a union of the objects
        below the first object in the first box, and those below the second object in the second box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box in objects_per_box:
            # max_y_loc corresponds to the bottom-most object.
            max_y_loc = max([obj.y_loc for obj in objects_per_box[box]])
            for candidate_obj in box.objects:
                if candidate_obj.y_loc > max_y_loc:
                    return_set.add(candidate_obj)
        return return_set

    @predicate
    def small(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.size == 10])

    @predicate
    def medium(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.size == 20])

    @predicate
    def big(self, objects: Set[Object]) -> Set[Object]:
        return set([obj for obj in objects if obj.size == 30])

    @predicate
    def box_count_equals(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) == count

    @predicate
    def box_count_not_equals(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) != count

    @predicate
    def box_count_greater(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) > count

    @predicate
    def box_count_greater_equals(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) >= count

    @predicate
    def box_count_lesser(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) < count

    @predicate
    def box_count_lesser_equals(self, boxes: Set[Box], count: int) -> bool:
        return len(boxes) <= count

    @predicate
    def object_color_all_equals(self, objects: Set[Object], color: Color) -> bool:
        return all([obj.color == color.color for obj in objects])

    @predicate
    def object_color_any_equals(self, objects: Set[Object], color: Color) -> bool:
        return any([obj.color == color.color for obj in objects])

    @predicate
    def object_color_none_equals(self, objects: Set[Object], color: Color) -> bool:
        return all([obj.color != color.color for obj in objects])

    @predicate
    def object_shape_all_equals(self, objects: Set[Object], shape: Shape) -> bool:
        return all([obj.shape == shape.shape for obj in objects])

    @predicate
    def object_shape_any_equals(self, objects: Set[Object], shape: Shape) -> bool:
        return any([obj.shape == shape.shape for obj in objects])

    @predicate
    def object_shape_none_equals(self, objects: Set[Object], shape: Shape) -> bool:
        return all([obj.shape != shape.shape for obj in objects])

    @predicate
    def object_count_equals(self, objects: Set[Object], count: int) -> bool:
        return len(objects) == count

    @predicate
    def object_count_not_equals(self, objects: Set[Object], count: int) -> bool:
        return len(objects) != count

    @predicate
    def object_count_greater(self, objects: Set[Object], count: int) -> bool:
        return len(objects) > count

    @predicate
    def object_count_greater_equals(self, objects: Set[Object], count: int) -> bool:
        return len(objects) >= count

    @predicate
    def object_count_lesser(self, objects: Set[Object], count: int) -> bool:
        return len(objects) < count

    @predicate
    def object_count_lesser_equals(self, objects: Set[Object], count: int) -> bool:
        return len(objects) <= count

    @predicate
    def object_color_count_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) == count

    @predicate
    def object_color_count_not_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) != count

    @predicate
    def object_color_count_greater(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) > count

    @predicate
    def object_color_count_greater_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) >= count

    @predicate
    def object_color_count_lesser(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) < count

    @predicate
    def object_color_count_lesser_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.color for obj in objects])) <= count

    @predicate
    def object_shape_count_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) == count

    @predicate
    def object_shape_count_not_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) != count

    @predicate
    def object_shape_count_greater(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) > count

    @predicate
    def object_shape_count_greater_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) >= count

    @predicate
    def object_shape_count_lesser(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) < count

    @predicate
    def object_shape_count_lesser_equals(self, objects: Set[Object], count: int) -> bool:
        return len(set([obj.shape for obj in objects])) <= count

    @predicate
    def member_count_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) == count])

    @predicate
    def member_count_not_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) != count])

    @predicate
    def member_count_greater(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) > count])

    @predicate
    def member_count_greater_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) >= count])

    @predicate
    def member_count_lesser(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) < count])

    @predicate
    def member_count_lesser_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.objects) <= count])

    @predicate
    def member_color_count_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) == count])

    @predicate
    def member_color_count_not_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) != count])

    @predicate
    def member_color_count_greater(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) > count])

    @predicate
    def member_color_count_greater_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) >= count])

    @predicate
    def member_color_count_lesser(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) < count])

    @predicate
    def member_color_count_lesser_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.colors) <= count])

    @predicate
    def member_shape_count_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) == count])

    @predicate
    def member_shape_count_not_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) != count])

    @predicate
    def member_shape_count_greater(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) > count])

    @predicate
    def member_shape_count_greater_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) >= count])

    @predicate
    def member_shape_count_lesser(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) < count])

    @predicate
    def member_shape_count_lesser_equals(self, boxes: Set[Box], count: int) -> Set[Box]:
        return set([box for box in boxes if len(box.shapes) <= count])

    @predicate
    def member_color_all_equals(self, boxes: Set[Box], color: Color) -> Set[Box]:
        return set([box for box in boxes if self.object_color_all_equals(box.objects, color)])

    @predicate
    def member_color_any_equals(self, boxes: Set[Box], color: Color) -> Set[Box]:
        return set([box for box in boxes if self.object_color_any_equals(box.objects, color)])

    @predicate
    def member_color_none_equals(self, boxes: Set[Box], color: Color) -> Set[Box]:
        return set([box for box in boxes if self.object_color_none_equals(box.objects, color)])

    @predicate
    def member_shape_all_equals(self, boxes: Set[Box], shape: Shape) -> Set[Box]:
        return set([box for box in boxes if self.object_shape_all_equals(box.objects, shape)])

    @predicate
    def member_shape_any_equals(self, boxes: Set[Box], shape: Shape) -> Set[Box]:
        return set([box for box in boxes if self.object_shape_any_equals(box.objects, shape)])

    @predicate
    def member_shape_none_equals(self, boxes: Set[Box], shape: Shape) -> Set[Box]:
        return set([box for box in boxes if self.object_shape_none_equals(box.objects, shape)])

    @predicate
    def member_shape_same(self, boxes: Set[Box]) -> Set[Box]:
        return set([box for box in boxes if self.object_shape_count_equals(box.objects, 1)])

    @predicate
    def member_color_same(self, boxes: Set[Box]) -> Set[Box]:
        return set([box for box in boxes if self.object_color_count_equals(box.objects, 1)])

    @predicate
    def member_shape_different(self, boxes: Set[Box]) -> Set[Box]:
        return set([box for box in boxes if self.object_shape_count_not_equals(box.objects, 1)])

    @predicate
    def member_color_different(self, boxes: Set[Box]) -> Set[Box]:
        return set([box for box in boxes if self.object_color_count_not_equals(box.objects, 1)])

    @predicate
    def negate_filter(self, filter_function: Callable[[Set[Object]], Set[Object]]) -> Callable[[Set[Object]],
                                                                                               Set[Object]]:
        def negated_filter(objects: Set[Object]) -> Set[Object]:
            return objects.difference(filter_function(objects))
        return negated_filter

    def _objects_touch_each_other(self, object1: Object, object2: Object) -> bool:
        """
        Returns true iff the objects touch each other.
        """
        in_vertical_range = object1.y_loc <= object2.y_loc + object2.size and \
                            object1.y_loc + object1.size >= object2.y_loc
        in_horizantal_range = object1.x_loc <= object2.x_loc + object2.size and \
                            object1.x_loc + object1.size >= object2.x_loc
        touch_side = object1.x_loc + object1.size == object2.x_loc or \
                     object2.x_loc + object2.size == object1.x_loc
        touch_top_or_bottom = object1.y_loc + object1.size == object2.y_loc or \
                              object2.y_loc + object2.size == object1.y_loc
        return (in_vertical_range and touch_side) or (in_horizantal_range and touch_top_or_bottom)

    def _separate_objects_by_boxes(self, objects: Set[Object]) -> Dict[Box, List[Object]]:
        """
        Given a set of objects, separate them by the boxes they belong to and return a dict.
        """
        objects_per_box: Dict[Box, List[Object]] = defaultdict(list)
        for box in self.boxes:
            for object_ in objects:
                if object_ in box.objects:
                    objects_per_box[box].append(object_)
        return objects_per_box

    def _get_objects_with_same_attribute(self,
                                         objects: Set[Object],
                                         attribute_function: Callable[[Object], str]) -> Set[Object]:
        """
        Returns the set of objects for which the attribute function returns an attribute value that
        is most frequent in the initial set, if the frequency is greater than 1. If not, all
        objects have different attribute values, and this method returns an empty set.
        """
        objects_of_attribute: Dict[str, Set[Object]] = defaultdict(set)
        for entity in objects:
            objects_of_attribute[attribute_function(entity)].add(entity)
        if not objects_of_attribute:
            return set()
        most_frequent_attribute = max(objects_of_attribute, key=lambda x: len(objects_of_attribute[x]))
        if len(objects_of_attribute[most_frequent_attribute]) <= 1:
            return set()
        return objects_of_attribute[most_frequent_attribute]



# pylint: disable=no-self-use,invalid-name
from allennlp.common.testing import AllenNlpTestCase

from allennlp.models.archival import load_archive
from allennlp.predictors import Predictor


class TestCorefPredictor(AllenNlpTestCase):
    def test_uses_named_inputs(self):
        inputs = {"document": "This is a single string document about a test. Sometimes it "
                              "contains coreferent parts."}
        archive = load_archive(self.FIXTURES_ROOT / 'coref' / 'serialization' / 'model.tar.gz')
        predictor = Predictor.from_archive(archive, 'coreference-resolution')

        result = predictor.predict_json(inputs)
        self.assert_predict_result(result)

        document = ['This', 'is', 'a', 'single', 'string',
                    'document', 'about', 'a', 'test', '.', 'Sometimes',
                    'it', 'contains', 'coreferent', 'parts', '.']

        result_doc_words = predictor.predict_tokenized(document)
        self.assert_predict_result(result_doc_words)

    @staticmethod
    def assert_predict_result(result):
        document = result["document"]
        assert document == ['This', 'is', 'a', 'single', 'string',
                            'document', 'about', 'a', 'test', '.', 'Sometimes',
                            'it', 'contains', 'coreferent', 'parts', '.']
        clusters = result["clusters"]
        assert isinstance(clusters, list)
        for cluster in clusters:
            assert isinstance(cluster, list)
            for mention in cluster:
                # Spans should be integer indices.
                assert isinstance(mention[0], int)
                assert isinstance(mention[1], int)
                # Spans should be inside document.
                assert 0 < mention[0] <= len(document)
                assert 0 < mention[1] <= len(document)



from typing import Callable, Dict, Generic, List, TypeVar

from allennlp.nn import util

ActionRepresentation = TypeVar('ActionRepresentation')  # pylint: disable=invalid-name


class GrammarStatelet(Generic[ActionRepresentation]):
    """
    A ``GrammarStatelet`` keeps track of the currently valid actions at every step of decoding.

    This class is relatively simple: we have a non-terminal stack which tracks which non-terminals
    we still need to expand.  At every timestep of decoding, we take an action that pops something
    off of the non-terminal stack, and possibly pushes more things on.  The grammar state is
    "finished" when the non-terminal stack is empty.

    At any point during decoding, you can query this object to get a representation of all of the
    valid actions in the current state.  The representation is something that you provide when
    constructing the initial state, in whatever form you want, and we just hold on to it for you
    and return it when you ask.  Putting this in here is purely for convenience, to group together
    pieces of state that are related to taking actions - if you want to handle the action
    representations outside of this class, that would work just fine too.

    Parameters
    ----------
    nonterminal_stack : ``List[str]``
        Holds the list of non-terminals that still need to be expanded.  This starts out as
        [START_SYMBOL], and decoding ends when this is empty.  Every time we take an action, we
        update the non-terminal stack and the context-dependent valid actions, and we use what's on
        the stack to decide which actions are valid in the current state.
    valid_actions : ``Dict[str, ActionRepresentation]``
        A mapping from non-terminals (represented as strings) to all valid expansions of that
        non-terminal.  The class that constructs this object can pick how it wants the actions to
        be represented.
    is_nonterminal : ``Callable[[str], bool]``
        A function that is used to determine whether each piece of the RHS of the action string is
        a non-terminal that needs to be added to the non-terminal stack.  You can use
        ``type_declaraction.is_nonterminal`` here, or write your own function if that one doesn't
        work for your domain.
    reverse_productions: ``bool``, optional (default=True)
        A flag that reverses the production rules when ``True``. If the production rules are
        reversed, then the first non-terminal in the production will be popped off the stack first,
        giving us left-to-right production.  If this is ``False``, you will get right-to-left
        production.
    """
    def __init__(self,
                 nonterminal_stack: List[str],
                 valid_actions: Dict[str, ActionRepresentation],
                 is_nonterminal: Callable[[str], bool],
                 reverse_productions: bool = True) -> None:
        self._nonterminal_stack = nonterminal_stack
        self._valid_actions = valid_actions
        self._is_nonterminal = is_nonterminal
        self._reverse_productions = reverse_productions

    def is_finished(self) -> bool:
        """
        Have we finished producing our logical form?  We have finished producing the logical form
        if and only if there are no more non-terminals on the stack.
        """
        return not self._nonterminal_stack

    def get_valid_actions(self) -> ActionRepresentation:
        """
        Returns the valid actions in the current grammar state.  The `Model` determines what
        exactly this looks like when it constructs the `valid_actions` dictionary.
        """
        return self._valid_actions[self._nonterminal_stack[-1]]

    def take_action(self, production_rule: str) -> 'GrammarStatelet':
        """
        Takes an action in the current grammar state, returning a new grammar state with whatever
        updates are necessary.  The production rule is assumed to be formatted as "LHS -> RHS".

        This will update the non-terminal stack.  Updating the non-terminal stack involves popping
        the non-terminal that was expanded off of the stack, then pushing on any non-terminals in
        the production rule back on the stack.

        For example, if our current ``nonterminal_stack`` is ``["r", "<e,r>", "d"]``, and
        ``action`` is ``d -> [<e,d>, e]``, the resulting stack will be ``["r", "<e,r>", "e",
        "<e,d>"]``.

        If ``self._reverse_productions`` is set to ``False`` then we push the non-terminals on in
        in their given order, which means that the first non-terminal in the production rule gets
        popped off the stack `last`.
        """
        left_side, right_side = production_rule.split(' -> ')
        assert self._nonterminal_stack[-1] == left_side, (f"Tried to expand {self._nonterminal_stack[-1]}"
                                                          f"but got rule {left_side} -> {right_side}")

        new_stack = self._nonterminal_stack[:-1]

        productions = self._get_productions_from_string(right_side)
        if self._reverse_productions:
            productions = list(reversed(productions))

        for production in productions:
            if self._is_nonterminal(production):
                new_stack.append(production)

        return GrammarStatelet(nonterminal_stack=new_stack,
                               valid_actions=self._valid_actions,
                               is_nonterminal=self._is_nonterminal,
                               reverse_productions=self._reverse_productions)

    @staticmethod
    def _get_productions_from_string(production_string: str) -> List[str]:
        """
        Takes a string like '[<d,d>, d]' and parses it into a list like ['<d,d>', 'd'].  For
        production strings that are not lists, like '<e,d>', we return a single-element list:
        ['<e,d>'].
        """
        if production_string[0] == '[':
            return production_string[1:-1].split(', ')
        else:
            return [production_string]

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            # pylint: disable=protected-access
            return all([
                    self._nonterminal_stack == other._nonterminal_stack,
                    util.tensors_equal(self._valid_actions, other._valid_actions),
                    self._is_nonterminal == other._is_nonterminal,
                    self._reverse_productions == other._reverse_productions,
                    ])
        return NotImplemented



# pylint: disable=no-self-use,invalid-name
import pytest

from allennlp.common import Params
from allennlp.common.util import ensure_list
from allennlp.data.dataset_readers import BabiReader
from allennlp.common.testing import AllenNlpTestCase


class TestBAbIReader:
    @pytest.mark.parametrize('keep_sentences, lazy', [(False, False), (False, True), (True, False), (True, True)])
    def test_read_from_file(self, keep_sentences, lazy):
        reader = BabiReader(keep_sentences=keep_sentences, lazy=lazy)
        instances = ensure_list(reader.read(AllenNlpTestCase.FIXTURES_ROOT / 'data' / 'babi.txt'))
        assert len(instances) == 8

        if keep_sentences:
            assert [t.text for t in instances[0].fields['context'][3].tokens[3:]] == ['of', 'wolves', '.']
            assert [t.sequence_index for t in instances[0].fields['supports']] == [0, 1]
        else:
            assert [t.text for t in instances[0].fields['context'].tokens[7:9]] == ['afraid', 'of']

    def test_can_build_from_params(self):
        reader = BabiReader.from_params(Params({'keep_sentences': True}))
        # pylint: disable=protected-access
        assert reader._keep_sentences
        assert reader._token_indexers['tokens'].__class__.__name__ == 'SingleIdTokenIndexer'



import json
from typing import Dict, List
import logging
from copy import deepcopy

from overrides import overrides
from parsimonious.exceptions import ParseError

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import Field, ArrayField, ListField, IndexField, \
        ProductionRuleField, TextField, MetadataField
from allennlp.data.instance import Instance
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenIndexer
from allennlp.data.tokenizers import Tokenizer, WordTokenizer
from allennlp.data.tokenizers.word_splitter import SpacyWordSplitter

from allennlp.semparse.worlds.atis_world import AtisWorld
from allennlp.semparse.contexts.atis_sql_table_context import NUMERIC_NONTERMINALS

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

END_OF_UTTERANCE_TOKEN = "@@EOU@@"

def _lazy_parse(text: str):
    for interaction in text.split("\n"):
        if interaction:
            yield json.loads(interaction)

@DatasetReader.register("atis")
class AtisDatasetReader(DatasetReader):
    # pylint: disable=line-too-long
    """
    This ``DatasetReader`` takes json files and converts them into ``Instances`` for the
    ``AtisSemanticParser``.

    Each line in the file is a JSON object that represent an interaction in the ATIS dataset
    that has the following keys and values:
    ```
    "id": The original filepath in the LDC corpus
    "interaction": <list where each element represents a turn in the interaction>
    "scenario": A code that refers to the scenario that served as the prompt for this interaction
    "ut_date": Date of the interaction
    "zc09_path": Path that was used in the original paper `Learning Context-Dependent Mappings from
    Sentences to Logical Form
    <https://www.semanticscholar.org/paper/Learning-Context-Dependent-Mappings-from-Sentences-Zettlemoyer-Collins/44a8fcee0741139fa15862dc4b6ce1e11444878f>'_ by Zettlemoyer and Collins (ACL/IJCNLP 2009)
    ```

    Each element in the ``interaction`` list has the following keys and values:
    ```
    "utterance": Natural language input
    "sql": A list of SQL queries that the utterance maps to, it could be multiple SQL queries
    or none at all.
    ```

    Parameters
    ----------
    token_indexers : ``Dict[str, TokenIndexer]``, optional
        Token indexers for the utterances. Will default to ``{"tokens": SingleIdTokenIndexer()}``.
    lazy : ``bool`` (optional, default=False)
        Passed to ``DatasetReader``.  If this is ``True``, training will start sooner, but will
        take longer per batch.
    tokenizer : ``Tokenizer``, optional
        Tokenizer to use for the utterances. Will default to ``WordTokenizer()`` with Spacy's tagger
        enabled.
    database_file: ``str``, optional
        The directory to find the sqlite database file. We query the sqlite database to find the strings
        that are allowed.
    num_turns_to_concatenate: ``str``, optional
        The number of utterances to concatenate as the conversation context.
    """
    def __init__(self,
                 token_indexers: Dict[str, TokenIndexer] = None,
                 keep_if_unparseable: bool = False,
                 lazy: bool = False,
                 tokenizer: Tokenizer = None,
                 database_file: str = None,
                 num_turns_to_concatenate: int = 1) -> None:
        super().__init__(lazy)
        self._keep_if_unparseable = keep_if_unparseable
        self._token_indexers = token_indexers or {'tokens': SingleIdTokenIndexer()}
        self._tokenizer = tokenizer or WordTokenizer(SpacyWordSplitter())
        self._database_file = database_file
        self._num_turns_to_concatenate = num_turns_to_concatenate

    @overrides
    def _read(self, file_path: str):
        # if `file_path` is a URL, redirect to the cache
        file_path = cached_path(file_path)

        with open(file_path) as atis_file:
            logger.info("Reading ATIS instances from dataset at : %s", file_path)
            for line in _lazy_parse(atis_file.read()):
                utterances = []
                for current_interaction in line['interaction']:
                    if not current_interaction['utterance'] or not current_interaction['sql']:
                        continue
                    utterances.append(current_interaction['utterance'])
                    sql_query_labels = [query for query in current_interaction['sql'].split('\n') if query]
                    instance = self.text_to_instance(deepcopy(utterances), sql_query_labels)
                    if not instance:
                        continue
                    yield instance

    @overrides
    def text_to_instance(self,  # type: ignore
                         utterances: List[str],
                         sql_query_labels: List[str] = None) -> Instance:
        # pylint: disable=arguments-differ
        """
        Parameters
        ----------
        utterances: ``List[str]``, required.
            List of utterances in the interaction, the last element is the current utterance.
        sql_query_labels: ``List[str]``, optional
            The SQL queries that are given as labels during training or validation.
        """
        if self._num_turns_to_concatenate:
            utterances[-1] = f' {END_OF_UTTERANCE_TOKEN} '.join(utterances[-self._num_turns_to_concatenate:])

        utterance = utterances[-1]
        action_sequence: List[str] = []

        if not utterance:
            return None

        world = AtisWorld(utterances=utterances)

        if sql_query_labels:
            # If there are multiple sql queries given as labels, we use the shortest
            # one for training.
            sql_query = min(sql_query_labels, key=len)
            try:
                action_sequence = world.get_action_sequence(sql_query)
            except ParseError:
                action_sequence = []
                logger.debug(f'Parsing error')

        tokenized_utterance = self._tokenizer.tokenize(utterance.lower())
        utterance_field = TextField(tokenized_utterance, self._token_indexers)

        production_rule_fields: List[Field] = []

        for production_rule in world.all_possible_actions():
            nonterminal, _ = production_rule.split(' ->')
            # The whitespaces are not semantically meaningful, so we filter them out.
            production_rule = ' '.join([token for token in production_rule.split(' ') if token != 'ws'])
            field = ProductionRuleField(production_rule, self._is_global_rule(nonterminal))
            production_rule_fields.append(field)

        action_field = ListField(production_rule_fields)
        action_map = {action.rule: i # type: ignore
                      for i, action in enumerate(action_field.field_list)}
        index_fields: List[Field] = []
        world_field = MetadataField(world)
        fields = {'utterance' : utterance_field,
                  'actions' : action_field,
                  'world' : world_field,
                  'linking_scores' : ArrayField(world.linking_scores)}

        if sql_query_labels != None:
            fields['sql_queries'] = MetadataField(sql_query_labels)
            if self._keep_if_unparseable or action_sequence:
                for production_rule in action_sequence:
                    index_fields.append(IndexField(action_map[production_rule], action_field))
                if not action_sequence:
                    index_fields = [IndexField(-1, action_field)]
                action_sequence_field = ListField(index_fields)
                fields['target_action_sequence'] = action_sequence_field
            else:
                # If we are given a SQL query, but we are unable to parse it, and we do not specify explicitly
                # to keep it, then we will skip the it.
                return None

        return Instance(fields)

    @staticmethod
    def _is_global_rule(nonterminal: str) -> bool:
        if nonterminal in NUMERIC_NONTERMINALS:
            return False
        elif nonterminal.endswith('string'):
            return False
        return True



from overrides import overrides

from allennlp.common.util import JsonDict
from allennlp.data import Instance
from allennlp.predictors.predictor import Predictor

@Predictor.register('event2mind')
class Event2MindPredictor(Predictor):
    """
    Predictor for the :class:`~allennlp.models.event2mind` model.
    """

    def predict(self, source: str) -> JsonDict:
        """
        Given a source string of some event, returns a JSON dictionary
        containing, for each target type, the top predicted sequences as
        indices, as tokens and the log probability of each.

        The JSON dictionary looks like:

        .. code-block:: js

            {
                `${target_type}_top_k_predictions`: [[1, 2, 3], [4, 5, 6], ...],
                `${target_type}_top_k_predicted_tokens`: [["to", "feel", "brave"], ...],
                `${target_type}_top_k_log_probabilities`: [-0.301, -0.046, ...]
            }

        By default ``target_type`` can be xreact, oreact and xintent.
        """
        return self.predict_json({"source" : source})

    @overrides
    def _json_to_instance(self, json_dict: JsonDict) -> Instance:
        """
        Expects JSON that looks like ``{"source": "..."}``.
        """
        source = json_dict["source"]
        return self._dataset_reader.text_to_instance(source)



from typing import Dict

import torch

from allennlp.common import Registrable

class TextFieldEmbedder(torch.nn.Module, Registrable):
    """
    A ``TextFieldEmbedder`` is a ``Module`` that takes as input the
    :class:`~allennlp.data.DataArray` produced by a :class:`~allennlp.data.fields.TextField` and
    returns as output an embedded representation of the tokens in that field.

    The ``DataArrays`` produced by ``TextFields`` are `dictionaries` with named representations,
    like "words" and "characters".  When you create a ``TextField``, you pass in a dictionary of
    :class:`~allennlp.data.TokenIndexer` objects, telling the field how exactly the tokens in the
    field should be represented.  This class changes the type signature of ``Module.forward``,
    restricting ``TextFieldEmbedders`` to take inputs corresponding to a single ``TextField``,
    which is a dictionary of tensors with the same names as were passed to the ``TextField``.

    We also add a method to the basic ``Module`` API: :func:`get_output_dim()`.  You might need
    this if you want to construct a ``Linear`` layer using the output of this embedder, for
    instance.
    """
    default_implementation = 'basic'

    def forward(self,  # pylint: disable=arguments-differ
                text_field_input: Dict[str, torch.Tensor],
                num_wrapping_dims: int = 0) -> torch.Tensor:
        """
        Parameters
        ----------
        text_field_input : ``Dict[str, torch.Tensor]``
            A dictionary that was the output of a call to ``TextField.as_tensor``.  Each tensor in
            here is assumed to have a shape roughly similar to ``(batch_size, sequence_length)``
            (perhaps with an extra trailing dimension for the characters in each token).
        num_wrapping_dims : ``int``, optional (default=0)
            If you have a ``ListField[TextField]`` that created the ``text_field_input``, you'll
            end up with tensors of shape ``(batch_size, wrapping_dim1, wrapping_dim2, ...,
            sequence_length)``.  This parameter tells us how many wrapping dimensions there are, so
            that we can correctly ``TimeDistribute`` the embedding of each named representation.
        """
        raise NotImplementedError

    def get_output_dim(self) -> int:
        """
        Returns the dimension of the vector representing each token in the output of this
        ``TextFieldEmbedder``.  This is `not` the shape of the returned tensor, but the last element of
        that shape.
        """
        raise NotImplementedError



#! /usr/bin/env python

# Script to launch AllenNLP Beaker jobs.

import argparse
import os
import json
import random
import tempfile
import subprocess
import sys

# This has to happen before we import spacy (even indirectly), because for some crazy reason spacy
# thought it was a good idea to set the random seed on import...
random_int = random.randint(0, 2**32)

sys.path.insert(0, os.path.dirname(os.path.abspath(os.path.join(os.path.join(__file__, os.pardir), os.pardir))))

from allennlp.common.params import Params


def main(param_file: str, args: argparse.Namespace):
    commit = subprocess.check_output(["git", "rev-parse", "HEAD"], universal_newlines=True).strip()
    image = f"allennlp/allennlp:{commit}"
    overrides = ""

    # Reads params and sets environment.
    ext_vars = {}

    for var in args.env:
        key, value = var.split("=")
        ext_vars[key] = value

    params = Params.from_file(param_file, overrides, ext_vars)
    flat_params = params.as_flat_dict()
    env = {}
    for k, v in flat_params.items():
        k = str(k).replace('.', '_')
        env[k] = str(v)

    # If the git repository is dirty, add a random hash.
    result = subprocess.run('git diff-index --quiet HEAD --', shell=True)
    if result.returncode != 0:
        dirty_hash = "%x" % random_int
        image += "-" + dirty_hash

    if args.blueprint:
        blueprint = args.blueprint
        print(f"Using the specified blueprint: {blueprint}")
    else:
        print(f"Building the Docker image ({image})...")
        subprocess.run(f'docker build -t {image} .', shell=True, check=True)

        print(f"Create a Beaker blueprint...")
        blueprint = subprocess.check_output(f'beaker blueprint create --quiet {image}', shell=True, universal_newlines=True).strip()
        print(f"  Blueprint created: {blueprint}")

    config_dataset_id = subprocess.check_output(f'beaker dataset create --quiet {param_file}', shell=True, universal_newlines=True).strip()

    allennlp_command = [
            "python",
            "-m",
            "allennlp.run",
            "train",
            "/config.json",
            "-s",
            "/output",
            "--file-friendly-logging"
        ]

    dataset_mounts = []
    for source in args.source + [f"{config_dataset_id}:/config.json"]:
        datasetId, containerPath = source.split(":")
        dataset_mounts.append({
            "datasetId": datasetId,
            "containerPath": containerPath
        })

    for var in args.env:
        key, value = var.split("=")
        env[key] = value

    requirements = {}
    if args.cpu:
        requirements["cpu"] = float(args.cpu)
    if args.memory:
        requirements["memory"] = args.memory
    if args.gpu_count:
        requirements["gpuCount"] = int(args.gpu_count)
    config_spec = {
        "description": args.desc,
        "blueprint": blueprint,
        "resultPath": "/output",
        "args": allennlp_command,
        "datasetMounts": dataset_mounts,
        "requirements": requirements,
        "env": env
    }
    config_task = {"spec": config_spec, "name": "training"}

    config = {
        "tasks": [config_task]
    }

    output_path = args.spec_output_path if args.spec_output_path else tempfile.mkstemp(".yaml",
            "beaker-config-")[1]
    with open(output_path, "w") as output:
        output.write(json.dumps(config, indent=4))
    print(f"Beaker spec written to {output_path}.")

    experiment_command = ["beaker", "experiment", "create", "--file", output_path]
    if args.name:
        experiment_command.append("--name")
        experiment_command.append(args.name.replace(" ", "-"))

    if args.dry_run:
        print(f"This is a dry run (--dry-run).  Launch your job with the following command:")
        print(f"    " + " ".join(experiment_command))
    else:
        print(f"Running the experiment:")
        print(f"    " + " ".join(experiment_command))
        subprocess.run(experiment_command)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument('param_file', type=str, help='The model configuration file.')
    parser.add_argument('--name', type=str, help='A name for the experiment.')
    parser.add_argument('--spec_output_path', type=str, help='The destination to write the experiment spec.')
    parser.add_argument('--dry-run', action='store_true', help='If specified, an experiment will not be created.')
    parser.add_argument('--blueprint', type=str, help='The Blueprint to use (if unspecified one will be built)')
    parser.add_argument('--desc', type=str, help='A description for the experiment.')
    parser.add_argument('--env', action='append', default=[], help='Set environment variables (e.g. NAME=value or NAME)')
    parser.add_argument('--source', action='append', default=[], help='Bind a remote data source (e.g. source-id:/target/path)')
    parser.add_argument('--cpu', help='CPUs to reserve for this experiment (e.g., 0.5)')
    parser.add_argument('--gpu-count', default=1, help='GPUs to use for this experiment (e.g., 1 (default))')
    parser.add_argument('--memory', help='Memory to reserve for this experiment (e.g., 1GB)')

    args = parser.parse_args()

    main(args.param_file, args)



from typing import List

import torch

from allennlp.common.checks import ConfigurationError
from allennlp.common.params import Params
from allennlp.models.model import Model
from allennlp.models.model import remove_pretrained_embedding_params

class Ensemble(Model):
    """
    An ensemble runs multiple instances of a model and selects an answer from the subresults via some
    ensembling strategy.

    Ensembles differ from most models in that they do not have a vocabulary or weights of their own
    (instead they rely on the vocabulary and weights from submodels).  Instead, the submodels are trained
    independently and the ensemble is created from the result.
    """
    # pylint: disable=abstract-method

    def __init__(self, submodels: List[Model]) -> None:
        vocab = submodels[0].vocab
        for submodel in submodels:
            if submodel.vocab != vocab:
                raise ConfigurationError("Vocabularies in ensemble differ")

        super(Ensemble, self).__init__(vocab, None)

        # Using ModuleList propagates calls to .eval() so dropout is disabled on the submodels in evaluation
        # and prediction.
        self.submodels = torch.nn.ModuleList(submodels)

    @classmethod
    def _load(cls,
              config: Params,
              serialization_dir: str,
              weights_file: str = None,
              cuda_device: int = -1) -> 'Model':
        """
        Ensembles don't have vocabularies or weights of their own, so they override _load.
        """
        model_params = config.get('model')

        # The experiment config tells us how to _train_ a model, including where to get pre-trained
        # embeddings from.  We're now _loading_ the model, so those embeddings will already be
        # stored in our weights.  We don't need any pretrained weight file anymore, and we don't
        # want the code to look for it, so we remove it from the parameters here.
        remove_pretrained_embedding_params(model_params)
        model = Model.from_params(vocab=None, params=model_params)

        # Force model to cpu or gpu, as appropriate, to make sure that the embeddings are
        # in sync with the weights
        if cuda_device >= 0:
            model.cuda(cuda_device)
        else:
            model.cpu()

        return model



from typing import List

from overrides import overrides
from spacy.tokens import Doc

from allennlp.common.util import JsonDict
from allennlp.common.util import get_spacy_model
from allennlp.data import DatasetReader, Instance
from allennlp.models import Model
from allennlp.predictors.predictor import Predictor


@Predictor.register("coreference-resolution")
class CorefPredictor(Predictor):
    """
    Predictor for the :class:`~allennlp.models.coreference_resolution.CoreferenceResolver` model.
    """
    def __init__(self, model: Model, dataset_reader: DatasetReader, language: str = 'en_core_web_sm') -> None:
        super().__init__(model, dataset_reader)

        # We have to use spacy to tokenise our document here, because we need
        # to also know sentence boundaries to propose valid mentions.
        self._spacy = get_spacy_model(language, pos_tags=True, parse=True, ner=False)

    def predict(self, document: str) -> JsonDict:
        """
        Predict the coreference clusters in the given document.

        .. code-block:: js

            {
            "document": [tokenised document text]
            "clusters":
              [
                [
                  [start_index, end_index],
                  [start_index, end_index]
                ],
                [
                  [start_index, end_index],
                  [start_index, end_index],
                  [start_index, end_index],
                ],
                ....
              ]
            }

        Parameters
        ----------
        document : ``str``
            A string representation of a document.

        Returns
        -------
        A dictionary representation of the predicted coreference clusters.
        """
        return self.predict_json({"document" : document})

    def predict_tokenized(self, tokenized_document: List[str]) -> JsonDict:
        """
        Predict the coreference clusters in the given document.

        Parameters
        ----------
        tokenized_document : ``List[str]``
            A list of words representation of a tokenized document.

        Returns
        -------
        A dictionary representation of the predicted coreference clusters.
        """
        instance = self._words_list_to_instance(tokenized_document)
        return self.predict_instance(instance)

    def _words_list_to_instance(self, words: List[str]) -> Instance:
        """
        Create an instance from words list represent an already tokenized document,
        for skipping tokenization when that information already exist for the user
        """
        spacy_document = Doc(self._spacy.vocab, words=words)
        for pipe in filter(None, self._spacy.pipeline):
            pipe[1](spacy_document)

        sentences = [[token.text for token in sentence] for sentence in spacy_document.sents]  # pylint: disable=not-an-iterable
        instance = self._dataset_reader.text_to_instance(sentences)
        return instance

    @overrides
    def _json_to_instance(self, json_dict: JsonDict) -> Instance:
        """
        Expects JSON that looks like ``{"document": "string of document text"}``
        """
        document = json_dict["document"]
        spacy_document = self._spacy(document)
        sentences = [[token.text for token in sentence] for sentence in spacy_document.sents]
        instance = self._dataset_reader.text_to_instance(sentences)
        return instance



# pylint: disable=no-self-use,invalid-name
import numpy
import pytest

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token
from allennlp.data.fields import TextField, SpanField
from allennlp.data.token_indexers import SingleIdTokenIndexer


class TestSpanField(AllenNlpTestCase):
    def setUp(self):
        super(TestSpanField, self).setUp()
        self.text = TextField([Token(t) for t in ["here", "is", "a", "sentence", "for", "spans", "."]],
                              {"words": SingleIdTokenIndexer("words")})

    def test_as_tensor_converts_span_field_correctly(self):
        span_field = SpanField(2, 3, self.text)
        tensor = span_field.as_tensor(span_field.get_padding_lengths()).detach().cpu().numpy()
        numpy.testing.assert_array_equal(tensor, numpy.array([2, 3]))

    def test_span_field_raises_on_incorrect_label_type(self):
        with pytest.raises(TypeError):
            _ = SpanField("hello", 3, self.text)

    def test_span_field_raises_on_ill_defined_span(self):
        with pytest.raises(ValueError):
            _ = SpanField(4, 1, self.text)

    def test_span_field_raises_if_span_end_is_greater_than_sentence_length(self):
        with pytest.raises(ValueError):
            _ = SpanField(1, 30, self.text)

    def test_empty_span_field_works(self):
        span_field = SpanField(1, 3, self.text)
        empty_span = span_field.empty_field()
        assert empty_span.span_start == -1
        assert empty_span.span_end == -1

    def test_printing_doesnt_crash(self):
        span_field = SpanField(2, 3, self.text)
        print(span_field)

    def test_equality(self):
        span_field1 = SpanField(2, 3, self.text)
        span_field2 = SpanField(2, 3, self.text)

        assert span_field1 == (2, 3)
        assert span_field1 == span_field1
        assert span_field1 != span_field2



# pylint: disable=invalid-name,no-self-use,protected-access
import argparse
import re
import shutil

import pytest
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.commands.fine_tune import FineTune, fine_tune_model_from_file_paths, \
                               fine_tune_model_from_args, fine_tune_model
from allennlp.common.params import Params
from allennlp.models import load_archive
from allennlp.modules.token_embedders.embedding import _read_pretrained_embeddings_file
from allennlp.data.vocabulary import Vocabulary
from allennlp.common.checks import ConfigurationError

class TestFineTune(AllenNlpTestCase):
    def setUp(self):
        super().setUp()
        self.model_archive = str(self.FIXTURES_ROOT / 'decomposable_attention' / 'serialization' / 'model.tar.gz')
        self.config_file = str(self.FIXTURES_ROOT / 'decomposable_attention' / 'experiment.json')
        self.serialization_dir = str(self.TEST_DIR / 'fine_tune')

        self.parser = argparse.ArgumentParser(description="Testing")
        subparsers = self.parser.add_subparsers(title='Commands', metavar='')
        FineTune().add_subparser('fine-tune', subparsers)


    def test_fine_tune_model_runs_from_file_paths(self):
        fine_tune_model_from_file_paths(model_archive_path=self.model_archive,
                                        config_file=self.config_file,
                                        serialization_dir=self.serialization_dir)

    def test_fine_tune_does_not_expand_vocab_by_default(self):
        params = Params.from_file(self.config_file)
        # snli2 has a new token in it
        params["train_data_path"] = str(self.FIXTURES_ROOT / 'data' / 'snli2.jsonl')

        model = load_archive(self.model_archive).model

        # By default, no vocab expansion.
        fine_tune_model(model, params, self.serialization_dir)

    def test_fine_tune_works_with_vocab_expansion(self):
        params = Params.from_file(self.config_file)
        # snli2 has a new token in it
        params["train_data_path"] = str(self.FIXTURES_ROOT / 'data' / 'snli2.jsonl')

        trained_model = load_archive(self.model_archive).model
        original_weight = trained_model._text_field_embedder.token_embedder_tokens.weight

        # If we do vocab expansion, we should not get error now.
        fine_tuned_model = fine_tune_model(trained_model, params, self.serialization_dir, extend_vocab=True)
        extended_weight = fine_tuned_model._text_field_embedder.token_embedder_tokens.weight

        assert tuple(original_weight.shape) == (24, 300)
        assert tuple(extended_weight.shape) == (25, 300)
        assert torch.all(original_weight == extended_weight[:24, :])

    def test_fine_tune_works_with_vocab_expansion_with_pretrained_file(self):
        params = Params.from_file(self.config_file)
        # snli2 has a new token (seahorse) in it
        params["train_data_path"] = str(self.FIXTURES_ROOT / 'data' / 'snli2.jsonl')

        # seahorse_embeddings.gz has only token embedding for 'seahorse'.
        embeddings_filename = str(self.FIXTURES_ROOT / 'data' / 'seahorse_embeddings.gz')
        extra_token_vector = _read_pretrained_embeddings_file(embeddings_filename, 300,
                                                              Vocabulary({"tokens": {"seahorse": 1}}))[2, :]
        unavailable_embeddings_filename = "file-not-found"

        def check_embedding_extension(user_pretrained_file, saved_pretrained_file, use_pretrained):
            trained_model = load_archive(self.model_archive).model
            original_weight = trained_model._text_field_embedder.token_embedder_tokens.weight
            # Simulate the behavior of unavailable pretrained_file being stored as an attribute.
            trained_model._text_field_embedder.token_embedder_tokens._pretrained_file = saved_pretrained_file
            embedding_sources_mapping = {"_text_field_embedder.token_embedder_tokens": user_pretrained_file}
            shutil.rmtree(self.serialization_dir, ignore_errors=True)
            fine_tuned_model = fine_tune_model(trained_model, params.duplicate(),
                                               self.serialization_dir, extend_vocab=True,
                                               embedding_sources_mapping=embedding_sources_mapping)
            extended_weight = fine_tuned_model._text_field_embedder.token_embedder_tokens.weight
            assert original_weight.shape[0] + 1 == extended_weight.shape[0] == 25
            assert torch.all(original_weight == extended_weight[:24, :])
            if use_pretrained:
                assert torch.all(extended_weight[24, :] == extra_token_vector)
            else:
                assert torch.all(extended_weight[24, :] != extra_token_vector)

        # TEST 1: Passing correct embedding_sources_mapping should work when pretrained_file attribute
        #         wasn't stored. (Model archive was generated without behaviour of storing pretrained_file)
        check_embedding_extension(embeddings_filename, None, True)

        # TEST 2: Passing correct embedding_sources_mapping should work when pretrained_file
        #         attribute was stored and user's choice should take precedence.
        check_embedding_extension(embeddings_filename, unavailable_embeddings_filename, True)

        # TEST 3: Passing no embedding_sources_mapping should work, if available pretrained_file
        #         attribute was stored.
        check_embedding_extension(None, embeddings_filename, True)

        # TEST 4: Passing incorrect pretrained-file by mapping should raise error.
        with pytest.raises(ConfigurationError):
            check_embedding_extension(unavailable_embeddings_filename, embeddings_filename, True)

        # TEST 5: If none is available, it should NOT raise error. Pretrained file could
        #         possibly not have been used in first place.
        check_embedding_extension(None, unavailable_embeddings_filename, False)

    def test_fine_tune_extended_model_is_loadable(self):
        params = Params.from_file(self.config_file)
        # snli2 has a new token (seahorse) in it
        params["train_data_path"] = str(self.FIXTURES_ROOT / 'data' / 'snli2.jsonl')
        trained_model = load_archive(self.model_archive).model
        shutil.rmtree(self.serialization_dir, ignore_errors=True)
        fine_tune_model(trained_model, params.duplicate(),
                        self.serialization_dir, extend_vocab=True)
        # self.serialization_dir = str(self.TEST_DIR / 'fine_tune')
        load_archive(str(self.TEST_DIR / 'fine_tune' / "model.tar.gz"))

    def test_fine_tune_runs_from_parser_arguments(self):
        raw_args = ["fine-tune",
                    "-m", self.model_archive,
                    "-c", self.config_file,
                    "-s", self.serialization_dir]

        args = self.parser.parse_args(raw_args)

        assert args.func == fine_tune_model_from_args
        assert args.model_archive == self.model_archive
        assert args.config_file == self.config_file
        assert args.serialization_dir == self.serialization_dir
        fine_tune_model_from_args(args)

    def test_fine_tune_fails_without_required_args(self):
        # Configuration file is required.
        with self.assertRaises(SystemExit) as context:
            self.parser.parse_args(["fine-tune", "-m", "path/to/archive", "-s", "serialization_dir"])
            assert context.exception.code == 2  # argparse code for incorrect usage

        # Serialization dir is required.
        with self.assertRaises(SystemExit) as context:
            self.parser.parse_args(["fine-tune", "-m", "path/to/archive", "-c", "path/to/config"])
            assert context.exception.code == 2  # argparse code for incorrect usage

        # Model archive is required.
        with self.assertRaises(SystemExit) as context:
            self.parser.parse_args(["fine-tune", "-s", "serialization_dir", "-c", "path/to/config"])
            assert context.exception.code == 2  # argparse code for incorrect usage

    def test_fine_tune_nograd_regex(self):
        original_model = load_archive(self.model_archive).model
        name_parameters_original = dict(original_model.named_parameters())
        regex_lists = [[],
                       [".*attend_feedforward.*", ".*token_embedder.*"],
                       [".*compare_feedforward.*"]]
        for regex_list in regex_lists:
            params = Params.from_file(self.config_file)
            params["trainer"]["no_grad"] = regex_list
            shutil.rmtree(self.serialization_dir, ignore_errors=True)
            tuned_model = fine_tune_model(model=original_model,
                                          params=params,
                                          serialization_dir=self.serialization_dir)
            # If regex is matched, parameter name should have requires_grad False
            # If regex is matched, parameter name should have same requires_grad
            # as the originally loaded model
            for name, parameter in tuned_model.named_parameters():
                if any(re.search(regex, name) for regex in regex_list):
                    assert not parameter.requires_grad
                else:
                    assert parameter.requires_grad \
                    == name_parameters_original[name].requires_grad
        # If all parameters have requires_grad=False, then error.
        with pytest.raises(Exception) as _:
            params = Params.from_file(self.config_file)
            params["trainer"]["no_grad"] = ["*"]
            shutil.rmtree(self.serialization_dir, ignore_errors=True)
            tuned_model = fine_tune_model(model=original_model,
                                          params=params,
                                          serialization_dir=self.serialization_dir)



# pylint: disable=no-self-use,invalid-name,protected-access
from copy import deepcopy

import numpy
from numpy.testing import assert_almost_equal
import torch

from allennlp.common import Params
from allennlp.data import Vocabulary
from allennlp.modules import Seq2VecEncoder
from allennlp.modules.token_embedders import Embedding, TokenCharactersEncoder
from allennlp.nn import InitializerApplicator, Initializer
from allennlp.common.testing import AllenNlpTestCase


class TestTokenCharactersEncoder(AllenNlpTestCase):
    def setUp(self):
        super(TestTokenCharactersEncoder, self).setUp()
        self.vocab = Vocabulary()
        self.vocab.add_token_to_namespace("1", "token_characters")
        self.vocab.add_token_to_namespace("2", "token_characters")
        self.vocab.add_token_to_namespace("3", "token_characters")
        self.vocab.add_token_to_namespace("4", "token_characters")
        params = Params({
                "embedding": {
                        "embedding_dim": 2,
                        "vocab_namespace": "token_characters"
                        },
                "encoder": {
                        "type": "cnn",
                        "embedding_dim": 2,
                        "num_filters": 4,
                        "ngram_filter_sizes": [1, 2],
                        "output_dim": 3
                        }
                })
        self.encoder = TokenCharactersEncoder.from_params(vocab=self.vocab, params=deepcopy(params))
        self.embedding = Embedding.from_params(vocab=self.vocab, params=params["embedding"])
        self.inner_encoder = Seq2VecEncoder.from_params(params["encoder"])
        constant_init = Initializer.from_params(Params({"type": "constant", "val": 1.}))
        initializer = InitializerApplicator([(".*", constant_init)])
        initializer(self.encoder)
        initializer(self.embedding)
        initializer(self.inner_encoder)

    def test_get_output_dim_uses_encoder_output_dim(self):
        assert self.encoder.get_output_dim() == 3

    def test_forward_applies_embedding_then_encoder(self):
        numpy_tensor = numpy.random.randint(6, size=(3, 4, 7))
        inputs = torch.from_numpy(numpy_tensor)
        encoder_output = self.encoder(inputs)
        reshaped_input = inputs.view(12, 7)
        embedded = self.embedding(reshaped_input)
        mask = (inputs != 0).long().view(12, 7)
        reshaped_manual_output = self.inner_encoder(embedded, mask)
        manual_output = reshaped_manual_output.view(3, 4, 3)
        assert_almost_equal(encoder_output.data.numpy(), manual_output.data.numpy())



# pylint: disable=no-self-use,invalid-name
import pytest

from allennlp.common import Params
from allennlp.common.checks import ConfigurationError
from allennlp.modules import Seq2VecEncoder
from allennlp.common.testing import AllenNlpTestCase


class TestSeq2VecEncoder(AllenNlpTestCase):
    def test_from_params_builders_encoder_correctly(self):
        # We're just making sure parameters get passed through correctly here, and that the basic
        # API works.
        params = Params({
                "type": "lstm",
                "bidirectional": True,
                "num_layers": 3,
                "input_size": 5,
                "hidden_size": 7
                })
        encoder = Seq2VecEncoder.from_params(params)
        # pylint: disable=protected-access
        assert encoder.__class__.__name__ == 'PytorchSeq2VecWrapper'
        assert encoder._module.__class__.__name__ == 'LSTM'
        assert encoder._module.num_layers == 3
        assert encoder._module.input_size == 5
        assert encoder._module.hidden_size == 7
        assert encoder._module.bidirectional is True
        assert encoder._module.batch_first is True

    def test_from_params_requires_batch_first(self):
        params = Params({
                "type": "lstm",
                "batch_first": False,
                })
        with pytest.raises(ConfigurationError):
            # pylint: disable=unused-variable
            encoder = Seq2VecEncoder.from_params(params)



# pylint: disable=no-self-use,invalid-name
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.modules.seq2seq_encoders import StackedSelfAttentionEncoder


class TestStackedSelfAttention(AllenNlpTestCase):
    def test_get_dimension_is_correct(self):
        encoder = StackedSelfAttentionEncoder(input_dim=9,
                                              hidden_dim=12,
                                              projection_dim=6,
                                              feedforward_hidden_dim=5,
                                              num_layers=3,
                                              num_attention_heads=3)
        assert encoder.get_input_dim() == 9
        # hidden_dim + projection_dim
        assert encoder.get_output_dim() == 12

    def test_stacked_self_attention_can_run_foward(self):
        # Correctness checks are elsewhere - this is just stacking
        # blocks which are already well tested, so we just check shapes.
        encoder = StackedSelfAttentionEncoder(input_dim=9,
                                              hidden_dim=12,
                                              projection_dim=9,
                                              feedforward_hidden_dim=5,
                                              num_layers=3,
                                              num_attention_heads=3)
        inputs = torch.randn([3, 5, 9])
        encoder_output = encoder(inputs, None)
        assert list(encoder_output.size()) == [3, 5, 12]



from allennlp.data.dataset_readers.dataset_utils.ontonotes import Ontonotes
from allennlp.data.dataset_readers.dataset_utils.ontonotes import OntonotesSentence
from allennlp.data.dataset_readers.dataset_utils.span_utils import enumerate_spans
from allennlp.data.dataset_readers.dataset_utils.span_utils import bio_tags_to_spans
from allennlp.data.dataset_readers.dataset_utils.span_utils import to_bioul, iob1_to_bioul
from allennlp.data.dataset_readers.dataset_utils.span_utils import bioul_tags_to_spans



# pylint: disable=no-self-use,invalid-name
from allennlp.common.testing import AllenNlpTestCase
from allennlp.semparse import util


class TestSemparseUtil(AllenNlpTestCase):
    def test_lisp_to_nested_expression(self):
        logical_form = "((reverse fb:row.row.year) (fb:row.row.league fb:cell.usl_a_league))"
        expression = util.lisp_to_nested_expression(logical_form)
        assert expression == [['reverse', 'fb:row.row.year'], ['fb:row.row.league', 'fb:cell.usl_a_league']]
        logical_form = "(count (and (division 1) (tier (!= null))))"
        expression = util.lisp_to_nested_expression(logical_form)
        assert expression == ['count', ['and', ['division', '1'], ['tier', ['!=', 'null']]]]



# pylint: disable=no-self-use,invalid-name,line-too-long

import json
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data.dataset_readers.dataset_utils import text2sql_utils

class Text2SqlUtilsTest(AllenNlpTestCase):

    def setUp(self):
        super().setUp()
        self.data = self.FIXTURES_ROOT / 'data' / 'text2sql' / 'restaurants_tiny.json'

    def test_process_sql_data_blob(self):

        data = json.load(open(str(self.data)))
        dataset = text2sql_utils.process_sql_data([data[0]])
        dataset = list(dataset)
        sql_data = dataset[0]
        # Check that question de-duplication happens by default
        # (otherwise there would be more than 1 dataset element).
        assert len(dataset) == 1
        assert sql_data.text == ['how', 'many', 'buttercup', 'kitchen', 'are', 'there', 'in', 'san', 'francisco', '?']
        assert sql_data.text_with_variables == ['how', 'many', 'name0', 'are', 'there', 'in', 'city_name0', '?']
        assert sql_data.sql == ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'LOCATION', 'AS', 'LOCATIONalias0', ',',
                                'RESTAURANT', 'AS', 'RESTAURANTalias0', 'WHERE', 'LOCATIONalias0', '.', 'CITY_NAME', '=',
                                '\'city_name0\'', 'AND', 'RESTAURANTalias0', '.', 'ID', '=', 'LOCATIONalias0', '.', 'RESTAURANT_ID',
                                'AND', 'RESTAURANTalias0', '.', 'NAME', '=', '\'name0\'', ';']

        assert sql_data.text_variables == {'city_name0': 'san francisco', 'name0': 'buttercup kitchen'}
        assert sql_data.sql_variables == {'city_name0': {'text': 'san francisco', 'type': 'city_name'},
                                          'name0': {'text': 'buttercup kitchen', 'type': 'name'}}


        dataset = text2sql_utils.process_sql_data([data[1]])
        correct_text = [
                [['how', 'many', 'chinese', 'restaurants', 'are', 'there', 'in', 'the', 'bay', 'area', '?'],
                 ['how', 'many', 'food_type0', 'restaurants', 'are', 'there', 'in', 'the', 'region0', '?']],
                [['how', 'many', 'places', 'for', 'chinese', 'food', 'are', 'there', 'in', 'the', 'bay', 'area', '?'],
                 ['how', 'many', 'places', 'for', 'food_type0', 'food', 'are', 'there', 'in', 'the', 'region0', '?']],
                [['how', 'many', 'chinese', 'places', 'are', 'there', 'in', 'the', 'bay', 'area', '?'],
                 ['how', 'many', 'food_type0', 'places', 'are', 'there', 'in', 'the', 'region0', '?']],
                [['how', 'many', 'places', 'for', 'chinese', 'are', 'there', 'in', 'the', 'bay', 'area', '?'],
                 ['how', 'many', 'places', 'for', 'food_type0', 'are', 'there', 'in', 'the', 'region0', '?']],
        ]

        for i, sql_data in enumerate(dataset):
            assert sql_data.sql == ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'GEOGRAPHIC', 'AS', 'GEOGRAPHICalias0',
                                    ',', 'RESTAURANT', 'AS', 'RESTAURANTalias0', 'WHERE', 'GEOGRAPHICalias0', '.', 'REGION',
                                    '=', '\'region0\'', 'AND', 'RESTAURANTalias0', '.', 'CITY_NAME', '=', 'GEOGRAPHICalias0',
                                    '.', 'CITY_NAME', 'AND', 'RESTAURANTalias0', '.', 'FOOD_TYPE', '=', '\'food_type0\'', ';']
            assert sql_data.text_variables == {'region0': 'bay area', 'food_type0': 'chinese'}
            assert sql_data.sql_variables == {'region0': {'text': 'bay area', 'type': 'region'},
                                              'food_type0': {'text': 'chinese', 'type': 'food_type'}}
            assert sql_data.text == correct_text[i][0]
            assert sql_data.text_with_variables == correct_text[i][1]

    def test_process_sql_data_can_yield_all_queries(self):
        data = json.load(open(str(self.data)))
        dataset = text2sql_utils.process_sql_data([data[0]], use_all_queries=True)
        dataset = list(dataset)
        assert len(dataset) == 3

    def test_replace_variables(self):
        sentence = ['how', 'many', 'name0', 'are', 'there', 'in', 'city_name0', '?']
        sentence_variables = {'city_name0': 'san francisco', 'name0': 'buttercup kitchen'}
        tokens, tags = text2sql_utils.replace_variables(sentence, sentence_variables)
        assert tokens == ['how', 'many', 'buttercup', 'kitchen', 'are', 'there', 'in', 'san', 'francisco', '?']
        assert tags == ['O', 'O', 'name0', 'name0', 'O', 'O', 'O', 'city_name0', 'city_name0', 'O']

    def test_clean_and_split_sql(self):
        sql = "SELECT COUNT( * ) FROM LOCATION AS LOCATIONalias0 , RESTAURANT AS RESTAURANTalias0 WHERE LOCATIONalias0.CITY_NAME " \
              "= \"city_name0\" AND RESTAURANTalias0.ID = LOCATIONalias0.RESTAURANT_ID AND RESTAURANTalias0.NAME = \"name0\" ;"

        cleaned = text2sql_utils.clean_and_split_sql(sql)
        assert cleaned == ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'LOCATION', 'AS', 'LOCATIONalias0', ',',
                           'RESTAURANT', 'AS', 'RESTAURANTalias0', 'WHERE', 'LOCATIONalias0', '.', 'CITY_NAME', '=',
                           '\'city_name0\'', 'AND', 'RESTAURANTalias0', '.', 'ID', '=', 'LOCATIONalias0', '.', 'RESTAURANT_ID',
                           'AND', 'RESTAURANTalias0', '.', 'NAME', '=', '\'name0\'', ';']


    def test_clean_unneeded_aliases(self):
        sql = ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'LOCATION', 'AS', 'LOCATIONalias0', ',',
               'RESTAURANT', 'AS', 'RESTAURANTalias0', 'WHERE', 'LOCATIONalias0', '.', 'CITY_NAME', '=',
               '\'city_name0\'', 'AND', 'RESTAURANTalias0', '.', 'ID', '=', 'LOCATIONalias0', '.', 'RESTAURANT_ID',
               'AND', 'RESTAURANTalias0', '.', 'NAME', '=', '\'name0\'', ';']

        cleaned = text2sql_utils.clean_unneeded_aliases(sql)
        assert cleaned == ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'LOCATION', ',', 'RESTAURANT', 'WHERE',
                           'LOCATION', '.', 'CITY_NAME', '=', "'city_name0'", 'AND', 'RESTAURANT', '.', 'ID',
                           '=', 'LOCATION', '.', 'RESTAURANT_ID', 'AND', 'RESTAURANT', '.', 'NAME', '=', "'name0'", ';']

        # Check we don't mangle decimal numbers:
        assert text2sql_utils.clean_unneeded_aliases(["2.5"]) == ["2.5"]

        # Check we don't remove non-trivial aliases:
        sql = ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'MAX', '(', 'LOCATION', '.', 'ID', ')', 'AS', 'LOCATIONalias0', ";"]
        assert text2sql_utils.clean_unneeded_aliases(sql) == sql

    def test_read_database_schema(self):
        schema = text2sql_utils.read_dataset_schema(self.FIXTURES_ROOT / 'data' / 'text2sql' / 'restaurants-schema.csv')
        # Make it easier to compare:
        schema = {k: [(x.name, x.column_type, x.is_primary_key) for x in v]
                  for k, v in schema.items()}
        assert schema == {
                'RESTAURANT': [
                        ('RESTAURANT_ID', 'int(11)', True),
                        ('NAME', 'varchar(255)', False),
                        ('FOOD_TYPE', 'varchar(255)', False),
                        ('CITY_NAME', 'varchar(255)', False),
                        ('RATING', '"decimal(1', False)
                ],
                'LOCATION': [
                        ('RESTAURANT_ID', 'int(11)', True),
                        ('HOUSE_NUMBER', 'int(11)', False),
                        ('STREET_NAME', 'varchar(255)', False),
                        ('CITY_NAME', 'varchar(255)', False)
                ],
                'GEOGRAPHIC': [
                        ('CITY_NAME', 'varchar(255)', True),
                        ('COUNTY', 'varchar(255)', False),
                        ('REGION', 'varchar(255)', False)]
                }

    def test_resolve_primary_keys_in_schema(self):
        schema = text2sql_utils.read_dataset_schema(self.FIXTURES_ROOT / 'data' / 'text2sql' / 'restaurants-schema.csv')
        sql = ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'MAX', '(', 'LOCATION', '.', 'ID', ')', 'AS', 'LOCATIONalias0', ";"]

        resolved = text2sql_utils.resolve_primary_keys_in_schema(sql, schema)
        assert resolved == ['SELECT', 'COUNT', '(', '*', ')', 'FROM', 'MAX', '(', 'LOCATION', '.', 'RESTAURANT_ID', ')', 'AS', 'LOCATIONalias0', ";"]


# pylint: disable=invalid-name
import pytest

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Vocabulary
from allennlp.common.params import Params
from allennlp.models.simple_tagger import SimpleTagger
from allennlp.data.dataset_readers import SequenceTaggingDatasetReader
from allennlp.training.optimizers import Optimizer
from allennlp.training import Trainer
from allennlp.data.iterators import BasicIterator


class TestOptimizer(AllenNlpTestCase):
    def setUp(self):
        super(TestOptimizer, self).setUp()
        self.instances = SequenceTaggingDatasetReader().read(self.FIXTURES_ROOT / 'data' / 'sequence_tagging.tsv')
        vocab = Vocabulary.from_instances(self.instances)
        self.model_params = Params({
                "text_field_embedder": {
                        "token_embedders": {
                                "tokens": {
                                        "type": "embedding",
                                        "embedding_dim": 5
                                        }
                                }
                        },
                "encoder": {
                        "type": "lstm",
                        "input_size": 5,
                        "hidden_size": 7,
                        "num_layers": 2
                        }
                })
        self.model = SimpleTagger.from_params(vocab=vocab, params=self.model_params)

    def test_optimizer_basic(self):
        optimizer_params = Params({
                "type": "sgd",
                "lr": 1
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        param_groups = optimizer.param_groups
        assert len(param_groups) == 1
        assert param_groups[0]['lr'] == 1

    def test_optimizer_parameter_groups(self):
        optimizer_params = Params({
                "type": "sgd",
                "lr": 1,
                "momentum": 5,
                "parameter_groups": [
                        # the repeated "bias_" checks a corner case
                        # NOT_A_VARIABLE_NAME displays a warning but does not raise an exception
                        [["weight_i", "bias_", "bias_", "NOT_A_VARIABLE_NAME"], {'lr': 2}],
                        [["tag_projection_layer"], {'lr': 3}],
                ]
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        param_groups = optimizer.param_groups

        assert len(param_groups) == 3
        assert param_groups[0]['lr'] == 2
        assert param_groups[1]['lr'] == 3
        # base case uses default lr
        assert param_groups[2]['lr'] == 1
        for k in range(3):
            assert param_groups[k]['momentum'] == 5

        # all LSTM parameters except recurrent connections (those with weight_h in name)
        assert len(param_groups[0]['params']) == 6
        # just the projection weight and bias
        assert len(param_groups[1]['params']) == 2
        # the embedding + recurrent connections left in the default group
        assert len(param_groups[2]['params']) == 3


    def test_parameter_type_inference(self):
        # Should work ok even with lr as a string
        optimizer_params = Params({
                "type": "sgd",
                "lr": "0.1"
        })

        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)

        assert optimizer.defaults["lr"] == 0.1

        # But should crash (in the Pytorch code) if we don't do the type inference
        optimizer_params = Params({
                "type": "sgd",
                "lr": "0.1",
                "infer_type_and_cast": False
        })

        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]

        with pytest.raises(TypeError):
            optimizer = Optimizer.from_params(parameters, optimizer_params)


class TestDenseSparseAdam(AllenNlpTestCase):

    def setUp(self):
        super(TestDenseSparseAdam, self).setUp()
        self.instances = SequenceTaggingDatasetReader().read(self.FIXTURES_ROOT / 'data' / 'sequence_tagging.tsv')
        self.vocab = Vocabulary.from_instances(self.instances)
        self.model_params = Params({
                "text_field_embedder": {
                        "token_embedders": {
                                "tokens": {
                                        "type": "embedding",
                                        "embedding_dim": 5,
                                        "sparse": True
                                        }
                                }
                        },
                "encoder": {
                        "type": "lstm",
                        "input_size": 5,
                        "hidden_size": 7,
                        "num_layers": 2
                        }
                })
        self.model = SimpleTagger.from_params(vocab=self.vocab, params=self.model_params)

    def test_can_optimise_model_with_dense_and_sparse_params(self):
        optimizer_params = Params({
                "type": "dense_sparse_adam"
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        iterator = BasicIterator(2)
        iterator.index_with(self.vocab)
        Trainer(self.model, optimizer, iterator, self.instances).train()
